# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Awaitable, Dict, List, Optional, Union

from scaleway_core.api import API
from scaleway_core.bridge import (
    Region,
    ScwFile,
    Zone,
    unmarshal_ScwFile,
)
from scaleway_core.utils import (
    WaitForOptions,
    fetch_all_pages_async,
    random_name,
    validate_path_param,
    wait_for_resource_async,
)
from .types import (
    CNI,
    ClusterStatus,
    Ingress,
    ListClustersRequestOrderBy,
    ListNodesRequestOrderBy,
    ListPoolsRequestOrderBy,
    NodeStatus,
    PoolStatus,
    PoolVolumeType,
    Runtime,
    Cluster,
    ClusterType,
    CreateClusterRequestAutoUpgrade,
    CreateClusterRequestAutoscalerConfig,
    CreateClusterRequestOpenIDConnectConfig,
    CreateClusterRequestPoolConfig,
    CreatePoolRequestUpgradePolicy,
    ExternalNode,
    ListClusterAvailableTypesResponse,
    ListClusterAvailableVersionsResponse,
    ListClusterTypesResponse,
    ListClustersResponse,
    ListNodesResponse,
    ListPoolsResponse,
    ListVersionsResponse,
    Node,
    Pool,
    UpdateClusterRequestAutoUpgrade,
    UpdateClusterRequestAutoscalerConfig,
    UpdateClusterRequestOpenIDConnectConfig,
    UpdatePoolRequestUpgradePolicy,
    Version,
    CreateClusterRequest,
    UpdateClusterRequest,
    UpgradeClusterRequest,
    SetClusterTypeRequest,
    MigrateToPrivateNetworkClusterRequest,
    CreatePoolRequest,
    UpgradePoolRequest,
    UpdatePoolRequest,
)
from .content import (
    CLUSTER_TRANSIENT_STATUSES,
    NODE_TRANSIENT_STATUSES,
    POOL_TRANSIENT_STATUSES,
)
from .marshalling import (
    marshal_CreateClusterRequest,
    marshal_CreatePoolRequest,
    marshal_MigrateToPrivateNetworkClusterRequest,
    marshal_SetClusterTypeRequest,
    marshal_UpdateClusterRequest,
    marshal_UpdatePoolRequest,
    marshal_UpgradeClusterRequest,
    marshal_UpgradePoolRequest,
    unmarshal_Cluster,
    unmarshal_Node,
    unmarshal_Pool,
    unmarshal_Version,
    unmarshal_ExternalNode,
    unmarshal_ListClusterAvailableTypesResponse,
    unmarshal_ListClusterAvailableVersionsResponse,
    unmarshal_ListClusterTypesResponse,
    unmarshal_ListClustersResponse,
    unmarshal_ListNodesResponse,
    unmarshal_ListPoolsResponse,
    unmarshal_ListVersionsResponse,
)


class K8SV1API(API):
    """
    Kubernetes API.

    Kubernetes API.
    """

    async def list_clusters(
        self,
        *,
        region: Optional[Region] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        order_by: ListClustersRequestOrderBy = ListClustersRequestOrderBy.CREATED_AT_ASC,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        name: Optional[str] = None,
        status: ClusterStatus = ClusterStatus.UNKNOWN,
        type_: Optional[str] = None,
    ) -> ListClustersResponse:
        """
        List Clusters.
        List all existing Kubernetes clusters in a specific region.
        :param region: Region to target. If none is passed will use default region from the config.
        :param organization_id: Organization ID on which to filter the returned clusters.
        :param project_id: Project ID on which to filter the returned clusters.
        :param order_by: Sort order of returned clusters.
        :param page: Page number to return for clusters, from the paginated results.
        :param page_size: Maximum number of clusters per page.
        :param name: Name to filter on, only clusters containing this substring in their name will be returned.
        :param status: Status to filter on, only clusters with this status will be returned.
        :param type_: Type to filter on, only clusters with this type will be returned.
        :return: :class:`ListClustersResponse <ListClustersResponse>`

        Usage:
        ::

            result = await api.list_clusters()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/clusters",
            params={
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
                "status": status,
                "type": type_,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListClustersResponse(res.json())

    async def list_clusters_all(
        self,
        *,
        region: Optional[Region] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        order_by: Optional[ListClustersRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        name: Optional[str] = None,
        status: Optional[ClusterStatus] = None,
        type_: Optional[str] = None,
    ) -> List[Cluster]:
        """
        List Clusters.
        List all existing Kubernetes clusters in a specific region.
        :param region: Region to target. If none is passed will use default region from the config.
        :param organization_id: Organization ID on which to filter the returned clusters.
        :param project_id: Project ID on which to filter the returned clusters.
        :param order_by: Sort order of returned clusters.
        :param page: Page number to return for clusters, from the paginated results.
        :param page_size: Maximum number of clusters per page.
        :param name: Name to filter on, only clusters containing this substring in their name will be returned.
        :param status: Status to filter on, only clusters with this status will be returned.
        :param type_: Type to filter on, only clusters with this type will be returned.
        :return: :class:`List[ListClustersResponse] <List[ListClustersResponse]>`

        Usage:
        ::

            result = await api.list_clusters_all()
        """

        return await fetch_all_pages_async(
            type=ListClustersResponse,
            key="clusters",
            fetcher=self.list_clusters,
            args={
                "region": region,
                "organization_id": organization_id,
                "project_id": project_id,
                "order_by": order_by,
                "page": page,
                "page_size": page_size,
                "name": name,
                "status": status,
                "type_": type_,
            },
        )

    async def create_cluster(
        self,
        *,
        type_: str,
        description: str,
        version: str,
        region: Optional[Region] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        name: Optional[str] = None,
        tags: Optional[List[str]] = None,
        cni: CNI = CNI.UNKNOWN_CNI,
        enable_dashboard: Optional[bool] = None,
        ingress: Optional[Ingress] = None,
        pools: Optional[List[CreateClusterRequestPoolConfig]] = None,
        autoscaler_config: Optional[CreateClusterRequestAutoscalerConfig] = None,
        auto_upgrade: Optional[CreateClusterRequestAutoUpgrade] = None,
        feature_gates: Optional[List[str]] = None,
        admission_plugins: Optional[List[str]] = None,
        open_id_connect_config: Optional[
            CreateClusterRequestOpenIDConnectConfig
        ] = None,
        apiserver_cert_sans: Optional[List[str]] = None,
        private_network_id: Optional[str] = None,
    ) -> Cluster:
        """
        Create a new Cluster.
        Create a new Kubernetes cluster in a Scaleway region.
        :param region: Region to target. If none is passed will use default region from the config.
        :param organization_id: Organization ID in which the cluster will be created.

        One-of ('project_identifier'): at most one of 'organization_id', 'project_id' could be set.
        :param project_id: Project ID in which the cluster will be created.

        One-of ('project_identifier'): at most one of 'organization_id', 'project_id' could be set.
        :param type_: Type of the cluster (possible values are kapsule, multicloud, kapsule-dedicated-8, kapsule-dedicated-16).
        :param name: Cluster name.
        :param description: Cluster description.
        :param tags: Tags associated with the cluster.
        :param version: Kubernetes version of the cluster.
        :param cni: Container Network Interface (CNI) plugin running in the cluster.
        :param enable_dashboard: Defines whether the Kubernetes Dashboard is enabled in the cluster.
        :param ingress: Ingress Controller running in the cluster (deprecated feature).
        :param pools: Pools created along with the cluster.
        :param autoscaler_config: Autoscaler configuration for the cluster. It allows you to set (to an extent) your preferred autoscaler configuration, which is an implementation of the cluster-autoscaler (https://github.com/kubernetes/autoscaler/tree/master/cluster-autoscaler/).
        :param auto_upgrade: Auto upgrade configuration of the cluster. This configuration enables to set a specific 2-hour time window in which the cluster can be automatically updated to the latest patch version.
        :param feature_gates: List of feature gates to enable.
        :param admission_plugins: List of admission plugins to enable.
        :param open_id_connect_config: OpenID Connect configuration of the cluster. This configuration enables to update the OpenID Connect configuration of the Kubernetes API server.
        :param apiserver_cert_sans: Additional Subject Alternative Names for the Kubernetes API server certificate.
        :param private_network_id: Private network ID for internal cluster communication (cannot be changed later).
        :return: :class:`Cluster <Cluster>`

        Usage:
        ::

            result = await api.create_cluster(
                type_="example",
                description="example",
                version="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/clusters",
            body=marshal_CreateClusterRequest(
                CreateClusterRequest(
                    type_=type_,
                    description=description,
                    version=version,
                    region=region,
                    organization_id=organization_id,
                    project_id=project_id,
                    name=name or random_name(prefix="k8s"),
                    tags=tags,
                    cni=cni,
                    enable_dashboard=enable_dashboard,
                    ingress=ingress,
                    pools=pools,
                    autoscaler_config=autoscaler_config,
                    auto_upgrade=auto_upgrade,
                    feature_gates=feature_gates,
                    admission_plugins=admission_plugins,
                    open_id_connect_config=open_id_connect_config,
                    apiserver_cert_sans=apiserver_cert_sans,
                    private_network_id=private_network_id,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Cluster(res.json())

    async def get_cluster(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
    ) -> Cluster:
        """
        Get a Cluster.
        Retrieve information about a specific Kubernetes cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: ID of the requested cluster.
        :return: :class:`Cluster <Cluster>`

        Usage:
        ::

            result = await api.get_cluster(cluster_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Cluster(res.json())

    async def wait_for_cluster(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[Cluster, Union[bool, Awaitable[bool]]]] = None,
    ) -> Cluster:
        """
        Waits for :class:`Cluster <Cluster>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: ID of the requested cluster.
        :param options: The options for the waiter
        :return: :class:`Cluster <Cluster>`

        Usage:
        ::

            result = api.wait_for_cluster(cluster_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in CLUSTER_TRANSIENT_STATUSES

        return await wait_for_resource_async(
            fetcher=self.get_cluster,
            options=options,
            args={
                "cluster_id": cluster_id,
                "region": region,
            },
        )

    async def update_cluster(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        tags: Optional[List[str]] = None,
        autoscaler_config: Optional[UpdateClusterRequestAutoscalerConfig] = None,
        enable_dashboard: Optional[bool] = None,
        ingress: Optional[Ingress] = None,
        auto_upgrade: Optional[UpdateClusterRequestAutoUpgrade] = None,
        feature_gates: Optional[List[str]] = None,
        admission_plugins: Optional[List[str]] = None,
        open_id_connect_config: Optional[
            UpdateClusterRequestOpenIDConnectConfig
        ] = None,
        apiserver_cert_sans: Optional[List[str]] = None,
    ) -> Cluster:
        """
        Update a Cluster.
        Update information on a specific Kubernetes cluster. You can update details such as its name, description, tags and configuration. To upgrade a cluster, you will need to use the dedicated endpoint.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: ID of the cluster to update.
        :param name: New external name for the cluster.
        :param description: New description for the cluster.
        :param tags: New tags associated with the cluster.
        :param autoscaler_config: New autoscaler config for the cluster.
        :param enable_dashboard: New value for the Kubernetes Dashboard enablement.
        :param ingress: New Ingress Controller for the cluster (deprecated feature).
        :param auto_upgrade: New auto upgrade configuration for the cluster. Note that all fields need to be set.
        :param feature_gates: List of feature gates to enable.
        :param admission_plugins: List of admission plugins to enable.
        :param open_id_connect_config: OpenID Connect configuration of the cluster. This configuration enables to update the OpenID Connect configuration of the Kubernetes API server.
        :param apiserver_cert_sans: Additional Subject Alternative Names for the Kubernetes API server certificate.
        :return: :class:`Cluster <Cluster>`

        Usage:
        ::

            result = await api.update_cluster(cluster_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "PATCH",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}",
            body=marshal_UpdateClusterRequest(
                UpdateClusterRequest(
                    cluster_id=cluster_id,
                    region=region,
                    name=name,
                    description=description,
                    tags=tags,
                    autoscaler_config=autoscaler_config,
                    enable_dashboard=enable_dashboard,
                    ingress=ingress,
                    auto_upgrade=auto_upgrade,
                    feature_gates=feature_gates,
                    admission_plugins=admission_plugins,
                    open_id_connect_config=open_id_connect_config,
                    apiserver_cert_sans=apiserver_cert_sans,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Cluster(res.json())

    async def delete_cluster(
        self,
        *,
        cluster_id: str,
        with_additional_resources: bool,
        region: Optional[Region] = None,
    ) -> Cluster:
        """
        Delete a Cluster.
        Delete a specific Kubernetes cluster and all its associated pools and nodes. Note that this method will not delete any Load Balancer or Block Volume that are associated with the cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: ID of the cluster to delete.
        :param with_additional_resources: Defines whether all volumes (including retain volume type), empty Private Networks and Load Balancers with a name starting with the cluster ID will also be deleted.
        :return: :class:`Cluster <Cluster>`

        Usage:
        ::

            result = await api.delete_cluster(
                cluster_id="example",
                with_additional_resources=True,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "DELETE",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}",
            params={
                "with_additional_resources": with_additional_resources,
            },
        )

        self._throw_on_error(res)
        return unmarshal_Cluster(res.json())

    async def upgrade_cluster(
        self,
        *,
        cluster_id: str,
        version: str,
        upgrade_pools: bool,
        region: Optional[Region] = None,
    ) -> Cluster:
        """
        Upgrade a Cluster.
        Upgrade a specific Kubernetes cluster and possibly its associated pools to a specific and supported Kubernetes version.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: ID of the cluster to upgrade.
        :param version: New Kubernetes version of the cluster. Note that the version should either be a higher patch version of the same minor version or the direct minor version after the current one.
        :param upgrade_pools: Defines whether pools will also be upgraded once the control plane is upgraded.
        :return: :class:`Cluster <Cluster>`

        Usage:
        ::

            result = await api.upgrade_cluster(
                cluster_id="example",
                version="example",
                upgrade_pools=True,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/upgrade",
            body=marshal_UpgradeClusterRequest(
                UpgradeClusterRequest(
                    cluster_id=cluster_id,
                    version=version,
                    upgrade_pools=upgrade_pools,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Cluster(res.json())

    async def set_cluster_type(
        self,
        *,
        cluster_id: str,
        type_: str,
        region: Optional[Region] = None,
    ) -> Cluster:
        """
        Change the Cluster type.
        Change the type of a specific Kubernetes cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: ID of the cluster to migrate from one type to another.
        :param type_: Type of the cluster. Note that some migrations are not possible (please refer to product documentation).
        :return: :class:`Cluster <Cluster>`

        Usage:
        ::

            result = await api.set_cluster_type(
                cluster_id="example",
                type_="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/set-type",
            body=marshal_SetClusterTypeRequest(
                SetClusterTypeRequest(
                    cluster_id=cluster_id,
                    type_=type_,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Cluster(res.json())

    async def list_cluster_available_versions(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
    ) -> ListClusterAvailableVersionsResponse:
        """
        List available versions for a Cluster.
        List the versions that a specific Kubernetes cluster is allowed to upgrade to. Results will include every patch version greater than the current patch, as well as one minor version ahead of the current version. Any upgrade skipping a minor version will not work.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: Cluster ID for which the available Kubernetes versions will be listed.
        :return: :class:`ListClusterAvailableVersionsResponse <ListClusterAvailableVersionsResponse>`

        Usage:
        ::

            result = await api.list_cluster_available_versions(cluster_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/available-versions",
        )

        self._throw_on_error(res)
        return unmarshal_ListClusterAvailableVersionsResponse(res.json())

    async def list_cluster_available_types(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
    ) -> ListClusterAvailableTypesResponse:
        """
        List available cluster types for a cluster.
        List the cluster types that a specific Kubernetes cluster is allowed to switch to.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: Cluster ID for which the available Kubernetes types will be listed.
        :return: :class:`ListClusterAvailableTypesResponse <ListClusterAvailableTypesResponse>`

        Usage:
        ::

            result = await api.list_cluster_available_types(cluster_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/available-types",
        )

        self._throw_on_error(res)
        return unmarshal_ListClusterAvailableTypesResponse(res.json())

    async def _get_cluster_kube_config(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
    ) -> Optional[ScwFile]:
        """
        Download the kubeconfig for a Cluster.
        Download the Kubernetes cluster config file (also known as `kubeconfig`) for a specific cluster to use it with `kubectl`.
        Tip: add `?dl=1` at the end of the URL to directly retrieve the base64 decoded kubeconfig. If you choose not to, the kubeconfig will be base64 encoded.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: Cluster ID for which to download the kubeconfig.
        :return: :class:`Optional[ScwFile] <Optional[ScwFile]>`

        Usage:
        ::

            result = await api._get_cluster_kube_config(cluster_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/kubeconfig",
        )

        self._throw_on_error(res)
        json = res.json()
        return unmarshal_ScwFile(json) if json is not None else None

    async def reset_cluster_admin_token(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
    ) -> Optional[None]:
        """
        Reset the admin token of a Cluster.
        Reset the admin token for a specific Kubernetes cluster. This will revoke the old admin token (which will not be usable afterwards) and create a new one. Note that you will need to download kubeconfig again to keep interacting with the cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: Cluster ID on which the admin token will be renewed.

        Usage:
        ::

            result = await api.reset_cluster_admin_token(cluster_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/reset-admin-token",
        )

        self._throw_on_error(res)
        return None

    async def migrate_to_private_network_cluster(
        self,
        *,
        cluster_id: str,
        private_network_id: str,
        region: Optional[Region] = None,
    ) -> Cluster:
        """
        Migrate an existing cluster to a Private Network cluster.
        Migrate a cluster that was created before the release of Private Network clusters to a new one with a Private Network.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: ID of the cluster to migrate.
        :param private_network_id: ID of the Private Network to link to the cluster.
        :return: :class:`Cluster <Cluster>`

        Usage:
        ::

            result = await api.migrate_to_private_network_cluster(
                cluster_id="example",
                private_network_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/migrate-to-private-network",
            body=marshal_MigrateToPrivateNetworkClusterRequest(
                MigrateToPrivateNetworkClusterRequest(
                    cluster_id=cluster_id,
                    private_network_id=private_network_id,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Cluster(res.json())

    async def list_pools(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
        order_by: ListPoolsRequestOrderBy = ListPoolsRequestOrderBy.CREATED_AT_ASC,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        name: Optional[str] = None,
        status: PoolStatus = PoolStatus.UNKNOWN,
    ) -> ListPoolsResponse:
        """
        List Pools in a Cluster.
        List all the existing pools for a specific Kubernetes cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: ID of the cluster whose pools will be listed.
        :param order_by: Sort order of returned pools.
        :param page: Page number for the returned pools.
        :param page_size: Maximum number of pools per page.
        :param name: Name to filter on, only pools containing this substring in their name will be returned.
        :param status: Status to filter on, only pools with this status will be returned.
        :return: :class:`ListPoolsResponse <ListPoolsResponse>`

        Usage:
        ::

            result = await api.list_pools(cluster_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/pools",
            params={
                "name": name,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "status": status,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListPoolsResponse(res.json())

    async def list_pools_all(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
        order_by: Optional[ListPoolsRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        name: Optional[str] = None,
        status: Optional[PoolStatus] = None,
    ) -> List[Pool]:
        """
        List Pools in a Cluster.
        List all the existing pools for a specific Kubernetes cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: ID of the cluster whose pools will be listed.
        :param order_by: Sort order of returned pools.
        :param page: Page number for the returned pools.
        :param page_size: Maximum number of pools per page.
        :param name: Name to filter on, only pools containing this substring in their name will be returned.
        :param status: Status to filter on, only pools with this status will be returned.
        :return: :class:`List[ListPoolsResponse] <List[ListPoolsResponse]>`

        Usage:
        ::

            result = await api.list_pools_all(cluster_id="example")
        """

        return await fetch_all_pages_async(
            type=ListPoolsResponse,
            key="pools",
            fetcher=self.list_pools,
            args={
                "cluster_id": cluster_id,
                "region": region,
                "order_by": order_by,
                "page": page,
                "page_size": page_size,
                "name": name,
                "status": status,
            },
        )

    async def create_pool(
        self,
        *,
        cluster_id: str,
        node_type: str,
        autoscaling: bool,
        size: int,
        container_runtime: Runtime,
        autohealing: bool,
        root_volume_type: PoolVolumeType,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        placement_group_id: Optional[str] = None,
        min_size: Optional[int] = None,
        max_size: Optional[int] = None,
        tags: Optional[List[str]] = None,
        kubelet_args: Optional[Dict[str, str]] = None,
        upgrade_policy: Optional[CreatePoolRequestUpgradePolicy] = None,
        zone: Optional[Zone] = None,
        root_volume_size: Optional[int] = None,
    ) -> Pool:
        """
        Create a new Pool in a Cluster.
        Create a new pool in a specific Kubernetes cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: Cluster ID to which the pool will be attached.
        :param name: Pool name.
        :param node_type: Node type is the type of Scaleway Instance wanted for the pool. Nodes with insufficient memory are not eligible (DEV1-S, PLAY2-PICO, STARDUST). 'external' is a special node type used to provision instances from other cloud providers in a Kosmos Cluster.
        :param placement_group_id: Placement group ID in which all the nodes of the pool will be created.
        :param autoscaling: Defines whether the autoscaling feature is enabled for the pool.
        :param size: Size (number of nodes) of the pool.
        :param min_size: Defines the minimum size of the pool. Note that this field is only used when autoscaling is enabled on the pool.
        :param max_size: Defines the maximum size of the pool. Note that this field is only used when autoscaling is enabled on the pool.
        :param container_runtime: Customization of the container runtime is available for each pool. Note that `docker` has been deprecated since version 1.20 and will be removed by version 1.24.
        :param autohealing: Defines whether the autohealing feature is enabled for the pool.
        :param tags: Tags associated with the pool.
        :param kubelet_args: Kubelet arguments to be used by this pool. Note that this feature is experimental.
        :param upgrade_policy: Pool upgrade policy.
        :param zone: Zone in which the pool's nodes will be spawned.
        :param root_volume_type: Defines the system volume disk type. Two different types of volume (`volume_type`) are provided: `l_ssd` is a local block storage which means your system is stored locally on your node's hypervisor. `b_ssd` is a remote block storage which means your system is stored on a centralized and resilient cluster.
        :param root_volume_size: System volume disk size.
        :return: :class:`Pool <Pool>`

        Usage:
        ::

            result = await api.create_pool(
                cluster_id="example",
                node_type="example",
                autoscaling=True,
                size=1,
                container_runtime=unknown_runtime,
                autohealing=True,
                root_volume_type=default_volume_type,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/pools",
            body=marshal_CreatePoolRequest(
                CreatePoolRequest(
                    cluster_id=cluster_id,
                    node_type=node_type,
                    autoscaling=autoscaling,
                    size=size,
                    container_runtime=container_runtime,
                    autohealing=autohealing,
                    root_volume_type=root_volume_type,
                    region=region,
                    name=name or random_name(prefix="pool"),
                    placement_group_id=placement_group_id,
                    min_size=min_size,
                    max_size=max_size,
                    tags=tags,
                    kubelet_args=kubelet_args,
                    upgrade_policy=upgrade_policy,
                    zone=zone,
                    root_volume_size=root_volume_size,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Pool(res.json())

    async def get_pool(
        self,
        *,
        pool_id: str,
        region: Optional[Region] = None,
    ) -> Pool:
        """
        Get a Pool in a Cluster.
        Retrieve details about a specific pool in a Kubernetes cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param pool_id: ID of the requested pool.
        :return: :class:`Pool <Pool>`

        Usage:
        ::

            result = await api.get_pool(pool_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_pool_id = validate_path_param("pool_id", pool_id)

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/pools/{param_pool_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Pool(res.json())

    async def wait_for_pool(
        self,
        *,
        pool_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[Pool, Union[bool, Awaitable[bool]]]] = None,
    ) -> Pool:
        """
        Waits for :class:`Pool <Pool>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param pool_id: ID of the requested pool.
        :param options: The options for the waiter
        :return: :class:`Pool <Pool>`

        Usage:
        ::

            result = api.wait_for_pool(pool_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in POOL_TRANSIENT_STATUSES

        return await wait_for_resource_async(
            fetcher=self.get_pool,
            options=options,
            args={
                "pool_id": pool_id,
                "region": region,
            },
        )

    async def upgrade_pool(
        self,
        *,
        pool_id: str,
        version: str,
        region: Optional[Region] = None,
    ) -> Pool:
        """
        Upgrade a Pool in a Cluster.
        Upgrade the Kubernetes version of a specific pool. Note that it only works if the targeted version matches the cluster's version.
        :param region: Region to target. If none is passed will use default region from the config.
        :param pool_id: ID of the pool to upgrade.
        :param version: New Kubernetes version for the pool.
        :return: :class:`Pool <Pool>`

        Usage:
        ::

            result = await api.upgrade_pool(
                pool_id="example",
                version="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_pool_id = validate_path_param("pool_id", pool_id)

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/pools/{param_pool_id}/upgrade",
            body=marshal_UpgradePoolRequest(
                UpgradePoolRequest(
                    pool_id=pool_id,
                    version=version,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Pool(res.json())

    async def update_pool(
        self,
        *,
        pool_id: str,
        region: Optional[Region] = None,
        autoscaling: Optional[bool] = None,
        size: Optional[int] = None,
        min_size: Optional[int] = None,
        max_size: Optional[int] = None,
        autohealing: Optional[bool] = None,
        tags: Optional[List[str]] = None,
        kubelet_args: Optional[Dict[str, str]] = None,
        upgrade_policy: Optional[UpdatePoolRequestUpgradePolicy] = None,
    ) -> Pool:
        """
        Update a Pool in a Cluster.
        Update the attributes of a specific pool, such as its desired size, autoscaling settings, and tags.
        :param region: Region to target. If none is passed will use default region from the config.
        :param pool_id: ID of the pool to update.
        :param autoscaling: New value for the pool autoscaling enablement.
        :param size: New desired pool size.
        :param min_size: New minimum size for the pool.
        :param max_size: New maximum size for the pool.
        :param autohealing: New value for the pool autohealing enablement.
        :param tags: New tags associated with the pool.
        :param kubelet_args: New Kubelet arguments to be used by this pool. Note that this feature is experimental.
        :param upgrade_policy: New upgrade policy for the pool.
        :return: :class:`Pool <Pool>`

        Usage:
        ::

            result = await api.update_pool(pool_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_pool_id = validate_path_param("pool_id", pool_id)

        res = self._request(
            "PATCH",
            f"/k8s/v1/regions/{param_region}/pools/{param_pool_id}",
            body=marshal_UpdatePoolRequest(
                UpdatePoolRequest(
                    pool_id=pool_id,
                    region=region,
                    autoscaling=autoscaling,
                    size=size,
                    min_size=min_size,
                    max_size=max_size,
                    autohealing=autohealing,
                    tags=tags,
                    kubelet_args=kubelet_args,
                    upgrade_policy=upgrade_policy,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Pool(res.json())

    async def delete_pool(
        self,
        *,
        pool_id: str,
        region: Optional[Region] = None,
    ) -> Pool:
        """
        Delete a Pool in a Cluster.
        Delete a specific pool from a cluster. Note that all the pool's nodes will also be deleted.
        :param region: Region to target. If none is passed will use default region from the config.
        :param pool_id: ID of the pool to delete.
        :return: :class:`Pool <Pool>`

        Usage:
        ::

            result = await api.delete_pool(pool_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_pool_id = validate_path_param("pool_id", pool_id)

        res = self._request(
            "DELETE",
            f"/k8s/v1/regions/{param_region}/pools/{param_pool_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Pool(res.json())

    async def create_external_node(
        self,
        *,
        pool_id: str,
        region: Optional[Region] = None,
    ) -> ExternalNode:
        """
        Create a Kosmos node.
        Retrieve metadata for a Kosmos node. This method is not intended to be called by end users but rather programmatically by the kapsule-node-agent.
        :param region: Region to target. If none is passed will use default region from the config.
        :param pool_id:
        :return: :class:`ExternalNode <ExternalNode>`

        Usage:
        ::

            result = await api.create_external_node(pool_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_pool_id = validate_path_param("pool_id", pool_id)

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/pools/{param_pool_id}/external-nodes",
        )

        self._throw_on_error(res)
        return unmarshal_ExternalNode(res.json())

    async def list_nodes(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
        pool_id: Optional[str] = None,
        order_by: ListNodesRequestOrderBy = ListNodesRequestOrderBy.CREATED_AT_ASC,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        name: Optional[str] = None,
        status: NodeStatus = NodeStatus.UNKNOWN,
    ) -> ListNodesResponse:
        """
        List Nodes in a Cluster.
        List all the existing nodes for a specific Kubernetes cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: Cluster ID from which the nodes will be listed from.
        :param pool_id: Pool ID on which to filter the returned nodes.
        :param order_by: Sort order of the returned nodes.
        :param page: Page number for the returned nodes.
        :param page_size: Maximum number of nodes per page.
        :param name: Name to filter on, only nodes containing this substring in their name will be returned.
        :param status: Status to filter on, only nodes with this status will be returned.
        :return: :class:`ListNodesResponse <ListNodesResponse>`

        Usage:
        ::

            result = await api.list_nodes(cluster_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_cluster_id = validate_path_param("cluster_id", cluster_id)

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/clusters/{param_cluster_id}/nodes",
            params={
                "name": name,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "pool_id": pool_id,
                "status": status,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListNodesResponse(res.json())

    async def list_nodes_all(
        self,
        *,
        cluster_id: str,
        region: Optional[Region] = None,
        pool_id: Optional[str] = None,
        order_by: Optional[ListNodesRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        name: Optional[str] = None,
        status: Optional[NodeStatus] = None,
    ) -> List[Node]:
        """
        List Nodes in a Cluster.
        List all the existing nodes for a specific Kubernetes cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :param cluster_id: Cluster ID from which the nodes will be listed from.
        :param pool_id: Pool ID on which to filter the returned nodes.
        :param order_by: Sort order of the returned nodes.
        :param page: Page number for the returned nodes.
        :param page_size: Maximum number of nodes per page.
        :param name: Name to filter on, only nodes containing this substring in their name will be returned.
        :param status: Status to filter on, only nodes with this status will be returned.
        :return: :class:`List[ListNodesResponse] <List[ListNodesResponse]>`

        Usage:
        ::

            result = await api.list_nodes_all(cluster_id="example")
        """

        return await fetch_all_pages_async(
            type=ListNodesResponse,
            key="nodes",
            fetcher=self.list_nodes,
            args={
                "cluster_id": cluster_id,
                "region": region,
                "pool_id": pool_id,
                "order_by": order_by,
                "page": page,
                "page_size": page_size,
                "name": name,
                "status": status,
            },
        )

    async def get_node(
        self,
        *,
        node_id: str,
        region: Optional[Region] = None,
    ) -> Node:
        """
        Get a Node in a Cluster.
        Retrieve details about a specific Kubernetes Node.
        :param region: Region to target. If none is passed will use default region from the config.
        :param node_id: ID of the requested node.
        :return: :class:`Node <Node>`

        Usage:
        ::

            result = await api.get_node(node_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_node_id = validate_path_param("node_id", node_id)

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/nodes/{param_node_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Node(res.json())

    async def wait_for_node(
        self,
        *,
        node_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[Node, Union[bool, Awaitable[bool]]]] = None,
    ) -> Node:
        """
        Waits for :class:`Node <Node>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param node_id: ID of the requested node.
        :param options: The options for the waiter
        :return: :class:`Node <Node>`

        Usage:
        ::

            result = api.wait_for_node(node_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in NODE_TRANSIENT_STATUSES

        return await wait_for_resource_async(
            fetcher=self.get_node,
            options=options,
            args={
                "node_id": node_id,
                "region": region,
            },
        )

    async def replace_node(
        self,
        *,
        node_id: str,
        region: Optional[Region] = None,
    ) -> Node:
        """
        Replace a Node in a Cluster.
        Replace a specific Node. The node will first be cordoned (scheduling will be disabled on it). The existing pods on the node will then be drained and rescheduled onto another schedulable node. Note that when there is not enough space to reschedule all the pods (such as in a one-node cluster), disruption of your applications can be expected.
        :param region: Region to target. If none is passed will use default region from the config.
        :param node_id: ID of the node to replace.
        :return: :class:`Node <Node>`
        :deprecated

        Usage:
        ::

            result = await api.replace_node(node_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_node_id = validate_path_param("node_id", node_id)

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/nodes/{param_node_id}/replace",
        )

        self._throw_on_error(res)
        return unmarshal_Node(res.json())

    async def reboot_node(
        self,
        *,
        node_id: str,
        region: Optional[Region] = None,
    ) -> Node:
        """
        Reboot a Node in a Cluster.
        Reboot a specific Node. The node will first be cordoned (scheduling will be disabled on it). The existing pods on the node will then be drained and rescheduled onto another schedulable node. Note that when there is not enough space to reschedule all the pods (such as in a one-node cluster), disruption of your applications can be expected.
        :param region: Region to target. If none is passed will use default region from the config.
        :param node_id: ID of the node to reboot.
        :return: :class:`Node <Node>`

        Usage:
        ::

            result = await api.reboot_node(node_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_node_id = validate_path_param("node_id", node_id)

        res = self._request(
            "POST",
            f"/k8s/v1/regions/{param_region}/nodes/{param_node_id}/reboot",
        )

        self._throw_on_error(res)
        return unmarshal_Node(res.json())

    async def delete_node(
        self,
        *,
        node_id: str,
        skip_drain: bool,
        replace: bool,
        region: Optional[Region] = None,
    ) -> Node:
        """
        Delete a Node in a Cluster.
        Delete a specific Node. Note that when there is not enough space to reschedule all the pods (such as in a one-node cluster), disruption of your applications can be expected.
        :param region: Region to target. If none is passed will use default region from the config.
        :param node_id: ID of the node to replace.
        :param skip_drain: Skip draining node from its workload.
        :param replace: Add a new node after the deletion of this node.
        :return: :class:`Node <Node>`

        Usage:
        ::

            result = await api.delete_node(
                node_id="example",
                skip_drain=True,
                replace=True,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_node_id = validate_path_param("node_id", node_id)

        res = self._request(
            "DELETE",
            f"/k8s/v1/regions/{param_region}/nodes/{param_node_id}",
            params={
                "replace": replace,
                "skip_drain": skip_drain,
            },
        )

        self._throw_on_error(res)
        return unmarshal_Node(res.json())

    async def list_versions(
        self,
        *,
        region: Optional[Region] = None,
    ) -> ListVersionsResponse:
        """
        List all available Versions.
        List all available versions for the creation of a new Kubernetes cluster.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`ListVersionsResponse <ListVersionsResponse>`

        Usage:
        ::

            result = await api.list_versions()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/versions",
        )

        self._throw_on_error(res)
        return unmarshal_ListVersionsResponse(res.json())

    async def get_version(
        self,
        *,
        version_name: str,
        region: Optional[Region] = None,
    ) -> Version:
        """
        Get a Version.
        Retrieve a specific Kubernetes version and its details.
        :param region: Region to target. If none is passed will use default region from the config.
        :param version_name: Requested version name.
        :return: :class:`Version <Version>`

        Usage:
        ::

            result = await api.get_version(version_name="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_version_name = validate_path_param("version_name", version_name)

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/versions/{param_version_name}",
        )

        self._throw_on_error(res)
        return unmarshal_Version(res.json())

    async def list_cluster_types(
        self,
        *,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListClusterTypesResponse:
        """
        List cluster types.
        List available cluster types and their technical details.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: Page number, from the paginated results, to return for cluster-types.
        :param page_size: Maximum number of clusters per page.
        :return: :class:`ListClusterTypesResponse <ListClusterTypesResponse>`

        Usage:
        ::

            result = await api.list_cluster_types()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/k8s/v1/regions/{param_region}/cluster-types",
            params={
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListClusterTypesResponse(res.json())

    async def list_cluster_types_all(
        self,
        *,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[ClusterType]:
        """
        List cluster types.
        List available cluster types and their technical details.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: Page number, from the paginated results, to return for cluster-types.
        :param page_size: Maximum number of clusters per page.
        :return: :class:`List[ListClusterTypesResponse] <List[ListClusterTypesResponse]>`

        Usage:
        ::

            result = await api.list_cluster_types_all()
        """

        return await fetch_all_pages_async(
            type=ListClusterTypesResponse,
            key="cluster_types",
            fetcher=self.list_cluster_types,
            args={
                "region": region,
                "page": page,
                "page_size": page_size,
            },
        )



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Region,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class ListCredentialsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    ID_ASC = "id_asc"
    ID_DESC = "id_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListNamespacesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"
    ID_ASC = "id_asc"
    ID_DESC = "id_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    PROJECT_ID_ASC = "project_id_asc"
    PROJECT_ID_DESC = "project_id_desc"

    def __str__(self) -> str:
        return str(self.value)


class NamespaceProtocol(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    NATS = "nats"
    SQS_SNS = "sqs_sns"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class Credential:
    """
    Credential.
    """

    id: str
    """
    ID of the credentials.
    """

    name: str
    """
    Name of the credentials.
    """

    namespace_id: str
    """
    Namespace containing the credentials.
    """

    protocol: NamespaceProtocol
    """
    Protocol associated with the credentials.
    """

    nats_credentials: Optional[CredentialNATSCredsFile]
    """
    Object containing the credentials, if the credentials are for a NATS namespace.
    
    One-of ('credential_type'): at most one of 'nats_credentials', 'sqs_sns_credentials' could be set.
    """

    sqs_sns_credentials: Optional[CredentialSQSSNSCreds]
    """
    Object containing the credentials and their metadata, if the credentials are for an SQS/SNS namespace.
    
    One-of ('credential_type'): at most one of 'nats_credentials', 'sqs_sns_credentials' could be set.
    """


@dataclass
class CredentialNATSCredsFile:
    """
    Credential.nats creds file.
    """

    content: str
    """
    Raw content of the NATS credentials file.
    """


@dataclass
class CredentialSQSSNSCreds:
    """
    Credential.sqssns creds.
    """

    access_key: str
    """
    Access key ID.
    """

    secret_key: Optional[str]
    """
    Secret key ID.
    """

    permissions: Optional[Permissions]
    """
    Permissions associated with these credentials.
    """


@dataclass
class CredentialSummary:
    """
    Credential summary.
    """

    id: str
    """
    ID of the credentials.
    """

    name: str
    """
    Name of the credentials.
    """

    namespace_id: str
    """
    Namespace containing the credentials.
    """

    protocol: NamespaceProtocol
    """
    Protocol associated with the credentials.
    """

    sqs_sns_credentials: Optional[CredentialSummarySQSSNSCreds]
    """
    Object containing the credentials and their metadata, if the credentials are for an SQS/SNS namespace.
    
    One-of ('credential_type'): at most one of 'sqs_sns_credentials' could be set.
    """


@dataclass
class CredentialSummarySQSSNSCreds:
    """
    Credential summary.sqssns creds.
    """

    access_key: str
    """
    Access key ID.
    """

    permissions: Optional[Permissions]
    """
    Permissions associated with these credentials.
    """


@dataclass
class ListCredentialsResponse:
    """
    List credentials response.
    """

    total_count: int
    """
    Total count of existing credentials (matching any filters specified).
    """

    credentials: List[CredentialSummary]
    """
    Credentials on this page.
    """


@dataclass
class ListNamespacesResponse:
    """
    List namespaces response.
    """

    total_count: int
    """
    Total count of existing namespaces (matching any filters specified).
    """

    namespaces: List[Namespace]
    """
    Namespaces on this page.
    """


@dataclass
class Namespace:
    """
    Namespace.
    """

    id: str
    """
    Namespace ID.
    """

    name: str
    """
    Namespace name.
    """

    endpoint: str
    """
    Endpoint of the service matching the namespace's protocol.
    """

    protocol: NamespaceProtocol
    """
    Namespace protocol.
    """

    project_id: str
    """
    Project ID of the Project containing the namespace.
    """

    region: Region
    """
    Region where the namespace is deployed.
    """

    created_at: Optional[datetime]
    """
    Namespace creation date.
    """

    updated_at: Optional[datetime]
    """
    Namespace last modification date.
    """


@dataclass
class Permissions:
    """
    Permissions.
    """

    can_publish: Optional[bool]
    """
    Defines whether the credentials bearer can publish messages to the service (send messages to SQS queues or publish to SNS topics).
    """

    can_receive: Optional[bool]
    """
    Defines whether the credentials bearer can receive messages from the service.
    """

    can_manage: Optional[bool]
    """
    Defines whether the credentials bearer can manage the associated resources (SQS queues or SNS topics or subscriptions).
    """


@dataclass
class ListNamespacesRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    organization_id: Optional[str]
    """
    Include only namespaces in this Organization.
    """

    project_id: Optional[str]
    """
    Include only namespaces in this Project.
    """

    page: Optional[int]
    """
    Page number to return.
    """

    page_size: Optional[int]
    """
    Maximum number of namespaces to return per page.
    """

    order_by: Optional[ListNamespacesRequestOrderBy]
    """
    Order in which to return results.
    """


@dataclass
class CreateNamespaceRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Namespace name.
    """

    protocol: Optional[NamespaceProtocol]
    """
    Namespace protocol. You must specify a valid protocol (and not `unknown`) to avoid an error.
    """

    project_id: Optional[str]
    """
    Project containing the Namespace.
    """


@dataclass
class UpdateNamespaceRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    namespace_id: str
    """
    ID of the Namespace to update.
    """

    name: Optional[str]
    """
    Namespace name.
    """


@dataclass
class GetNamespaceRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    namespace_id: str
    """
    ID of the Namespace to get.
    """


@dataclass
class DeleteNamespaceRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    namespace_id: str
    """
    ID of the namespace to delete.
    """


@dataclass
class CreateCredentialRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    namespace_id: str
    """
    Namespace containing the credentials.
    """

    name: Optional[str]
    """
    Name of the credentials.
    """

    permissions: Optional[Permissions]
    """
    Permissions associated with these credentials.
    """


@dataclass
class DeleteCredentialRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    credential_id: str
    """
    ID of the credentials to delete.
    """


@dataclass
class ListCredentialsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    namespace_id: Optional[str]
    """
    Namespace containing the credentials.
    """

    page: Optional[int]
    """
    Page number to return.
    """

    page_size: Optional[int]
    """
    Maximum number of credentials to return per page.
    """

    order_by: Optional[ListCredentialsRequestOrderBy]
    """
    Order in which to return results.
    """


@dataclass
class UpdateCredentialRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    credential_id: str
    """
    ID of the credentials to update.
    """

    name: Optional[str]
    """
    Name of the credentials.
    """

    permissions: Optional[Permissions]
    """
    Permissions associated with these credentials.
    """


@dataclass
class GetCredentialRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    credential_id: str
    """
    ID of the credentials to get.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from datetime import datetime
from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    ScwFile,
    unmarshal_ScwFile,
)
from scaleway_core.utils import (
    fetch_all_pages_async,
    validate_path_param,
)
from .types import (
    DownloadInvoiceRequestFileType,
    InvoiceType,
    ListInvoicesRequestOrderBy,
    GetConsumptionResponse,
    Invoice,
    ListInvoicesResponse,
)
from .marshalling import (
    unmarshal_GetConsumptionResponse,
    unmarshal_ListInvoicesResponse,
)


class BillingV2Alpha1API(API):
    """
    Billing API.

    This API allows you to query your consumption.
    Billing API.
    """

    async def get_consumption(
        self,
        *,
        organization_id: Optional[str] = None,
    ) -> GetConsumptionResponse:
        """

        Usage:
        ::

            result = await api.get_consumption()
        """

        res = self._request(
            "GET",
            f"/billing/v2alpha1/consumption",
            params={
                "organization_id": organization_id
                or self.client.default_organization_id,
            },
        )

        self._throw_on_error(res)
        return unmarshal_GetConsumptionResponse(res.json())

    async def list_invoices(
        self,
        *,
        organization_id: Optional[str] = None,
        started_after: Optional[datetime] = None,
        started_before: Optional[datetime] = None,
        invoice_type: InvoiceType = InvoiceType.UNKNOWN_TYPE,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: ListInvoicesRequestOrderBy = ListInvoicesRequestOrderBy.INVOICE_NUMBER_DESC,
    ) -> ListInvoicesResponse:
        """

        Usage:
        ::

            result = await api.list_invoices()
        """

        res = self._request(
            "GET",
            f"/billing/v2alpha1/invoices",
            params={
                "invoice_type": invoice_type,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "started_after": started_after,
                "started_before": started_before,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListInvoicesResponse(res.json())

    async def list_invoices_all(
        self,
        *,
        organization_id: Optional[str] = None,
        started_after: Optional[datetime] = None,
        started_before: Optional[datetime] = None,
        invoice_type: Optional[InvoiceType] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListInvoicesRequestOrderBy] = None,
    ) -> List[Invoice]:
        """
        :return: :class:`List[ListInvoicesResponse] <List[ListInvoicesResponse]>`

        Usage:
        ::

            result = await api.list_invoices_all()
        """

        return await fetch_all_pages_async(
            type=ListInvoicesResponse,
            key="invoices",
            fetcher=self.list_invoices,
            args={
                "organization_id": organization_id,
                "started_after": started_after,
                "started_before": started_before,
                "invoice_type": invoice_type,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
            },
        )

    async def download_invoice(
        self,
        *,
        invoice_id: str,
        file_type: DownloadInvoiceRequestFileType,
    ) -> Optional[ScwFile]:
        """

        Usage:
        ::

            result = await api.download_invoice(
                invoice_id="example",
                file_type=pdf,
            )
        """

        param_invoice_id = validate_path_param("invoice_id", invoice_id)

        res = self._request(
            "GET",
            f"/billing/v2alpha1/invoices/{param_invoice_id}/download",
            params={
                "file_type": file_type,
            },
        )

        self._throw_on_error(res)
        json = res.json()
        return unmarshal_ScwFile(json) if json is not None else None



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import DeviceMessageFiltersRulePolicy
from .types import DeviceStatus
from .types import HubProductPlan
from .types import HubStatus
from .types import ListDevicesRequestOrderBy
from .types import ListHubsRequestOrderBy
from .types import ListNetworksRequestOrderBy
from .types import ListRoutesRequestOrderBy
from .types import NetworkNetworkType
from .types import NullValue
from .types import RouteDatabaseConfigEngine
from .types import RouteRestConfigHttpVerb
from .types import RouteRouteType
from .types import RouteS3ConfigS3Strategy
from .types import Certificate
from .types import CreateDeviceResponse
from .types import CreateNetworkResponse
from .types import CreateRouteRequestDatabaseConfig
from .types import CreateRouteRequestRestConfig
from .types import CreateRouteRequestS3Config
from .types import Device
from .types import DeviceMessageFilters
from .types import DeviceMessageFiltersRule
from .types import GetDeviceCertificateResponse
from .types import GetDeviceMetricsResponse
from .types import GetHubCAResponse
from .types import GetHubMetricsResponse
from .types import Hub
from .types import HubTwinsGraphiteConfig
from .types import ListDevicesResponse
from .types import ListHubsResponse
from .types import ListNetworksResponse
from .types import ListRoutesResponse
from .types import ListTwinDocumentsResponse
from .types import ListTwinDocumentsResponseDocumentSummary
from .types import Network
from .types import RenewDeviceCertificateResponse
from .types import Route
from .types import RouteDatabaseConfig
from .types import RouteRestConfig
from .types import RouteS3Config
from .types import RouteSummary
from .types import SetDeviceCertificateResponse
from .types import TwinDocument
from .types import UpdateRouteRequestDatabaseConfig
from .types import UpdateRouteRequestRestConfig
from .types import UpdateRouteRequestS3Config
from .content import HUB_TRANSIENT_STATUSES
from .api import IotV1API

__all__ = [
    "DeviceMessageFiltersRulePolicy",
    "DeviceStatus",
    "HubProductPlan",
    "HubStatus",
    "ListDevicesRequestOrderBy",
    "ListHubsRequestOrderBy",
    "ListNetworksRequestOrderBy",
    "ListRoutesRequestOrderBy",
    "NetworkNetworkType",
    "NullValue",
    "RouteDatabaseConfigEngine",
    "RouteRestConfigHttpVerb",
    "RouteRouteType",
    "RouteS3ConfigS3Strategy",
    "Certificate",
    "CreateDeviceResponse",
    "CreateNetworkResponse",
    "CreateRouteRequestDatabaseConfig",
    "CreateRouteRequestRestConfig",
    "CreateRouteRequestS3Config",
    "Device",
    "DeviceMessageFilters",
    "DeviceMessageFiltersRule",
    "GetDeviceCertificateResponse",
    "GetDeviceMetricsResponse",
    "GetHubCAResponse",
    "GetHubMetricsResponse",
    "Hub",
    "HubTwinsGraphiteConfig",
    "ListDevicesResponse",
    "ListHubsResponse",
    "ListNetworksResponse",
    "ListRoutesResponse",
    "ListTwinDocumentsResponse",
    "ListTwinDocumentsResponseDocumentSummary",
    "Network",
    "RenewDeviceCertificateResponse",
    "Route",
    "RouteDatabaseConfig",
    "RouteRestConfig",
    "RouteS3Config",
    "RouteSummary",
    "SetDeviceCertificateResponse",
    "TwinDocument",
    "UpdateRouteRequestDatabaseConfig",
    "UpdateRouteRequestRestConfig",
    "UpdateRouteRequestS3Config",
    "HUB_TRANSIENT_STATUSES",
    "IotV1API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    HumanStatus,
)


HUMAN_TRANSIENT_STATUSES: List[HumanStatus] = [
    HumanStatus.RUNNING,
]
"""
Lists transient statutes of the enum :class:`HumanStatus <HumanStatus>`.
"""



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    ClusterStatus,
    NodeStatus,
    PoolStatus,
)


CLUSTER_TRANSIENT_STATUSES: List[ClusterStatus] = [
    ClusterStatus.CREATING,
    ClusterStatus.DELETING,
    ClusterStatus.UPDATING,
]
"""
Lists transient statutes of the enum :class:`ClusterStatus <ClusterStatus>`.
"""

NODE_TRANSIENT_STATUSES: List[NodeStatus] = [
    NodeStatus.CREATING,
    NodeStatus.DELETING,
    NodeStatus.REBOOTING,
    NodeStatus.UPGRADING,
    NodeStatus.STARTING,
    NodeStatus.REGISTERING,
]
"""
Lists transient statutes of the enum :class:`NodeStatus <NodeStatus>`.
"""

POOL_TRANSIENT_STATUSES: List[PoolStatus] = [
    PoolStatus.DELETING,
    PoolStatus.SCALING,
    PoolStatus.UPGRADING,
]
"""
Lists transient statutes of the enum :class:`PoolStatus <PoolStatus>`.
"""



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    Region,
)
from scaleway_core.utils import (
    fetch_all_pages_async,
    random_name,
    validate_path_param,
)
from .types import (
    ListCredentialsRequestOrderBy,
    ListNamespacesRequestOrderBy,
    NamespaceProtocol,
    Credential,
    CredentialSummary,
    ListCredentialsResponse,
    ListNamespacesResponse,
    Namespace,
    Permissions,
    CreateNamespaceRequest,
    UpdateNamespaceRequest,
    CreateCredentialRequest,
    UpdateCredentialRequest,
)
from .marshalling import (
    marshal_CreateCredentialRequest,
    marshal_CreateNamespaceRequest,
    marshal_UpdateCredentialRequest,
    marshal_UpdateNamespaceRequest,
    unmarshal_Namespace,
    unmarshal_Credential,
    unmarshal_ListCredentialsResponse,
    unmarshal_ListNamespacesResponse,
)


class MnqV1Alpha1API(API):
    """
    Messaging and Queuing API.

    This API allows you to manage Scaleway Messaging and Queueing brokers.
    Messaging and Queuing API.
    """

    async def list_namespaces(
        self,
        *,
        region: Optional[Region] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: ListNamespacesRequestOrderBy = ListNamespacesRequestOrderBy.CREATED_AT_ASC,
    ) -> ListNamespacesResponse:
        """
        List namespaces.
        List all Messaging and Queuing namespaces in the specified region, for a Scaleway Organization or Project. By default, the namespaces returned in the list are ordered by creation date in ascending order, though this can be modified via the `order_by` field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param organization_id: Include only namespaces in this Organization.
        :param project_id: Include only namespaces in this Project.
        :param page: Page number to return.
        :param page_size: Maximum number of namespaces to return per page.
        :param order_by: Order in which to return results.
        :return: :class:`ListNamespacesResponse <ListNamespacesResponse>`

        Usage:
        ::

            result = await api.list_namespaces()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/mnq/v1alpha1/regions/{param_region}/namespaces",
            params={
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListNamespacesResponse(res.json())

    async def list_namespaces_all(
        self,
        *,
        region: Optional[Region] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListNamespacesRequestOrderBy] = None,
    ) -> List[Namespace]:
        """
        List namespaces.
        List all Messaging and Queuing namespaces in the specified region, for a Scaleway Organization or Project. By default, the namespaces returned in the list are ordered by creation date in ascending order, though this can be modified via the `order_by` field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param organization_id: Include only namespaces in this Organization.
        :param project_id: Include only namespaces in this Project.
        :param page: Page number to return.
        :param page_size: Maximum number of namespaces to return per page.
        :param order_by: Order in which to return results.
        :return: :class:`List[ListNamespacesResponse] <List[ListNamespacesResponse]>`

        Usage:
        ::

            result = await api.list_namespaces_all()
        """

        return await fetch_all_pages_async(
            type=ListNamespacesResponse,
            key="namespaces",
            fetcher=self.list_namespaces,
            args={
                "region": region,
                "organization_id": organization_id,
                "project_id": project_id,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
            },
        )

    async def create_namespace(
        self,
        *,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        protocol: NamespaceProtocol = NamespaceProtocol.UNKNOWN,
        project_id: Optional[str] = None,
    ) -> Namespace:
        """
        Create a namespace.
        Create a Messaging and Queuing namespace, set to the desired protocol.
        :param region: Region to target. If none is passed will use default region from the config.
        :param name: Namespace name.
        :param protocol: Namespace protocol. You must specify a valid protocol (and not `unknown`) to avoid an error.
        :param project_id: Project containing the Namespace.
        :return: :class:`Namespace <Namespace>`

        Usage:
        ::

            result = await api.create_namespace()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/mnq/v1alpha1/regions/{param_region}/namespaces",
            body=marshal_CreateNamespaceRequest(
                CreateNamespaceRequest(
                    region=region,
                    name=name or random_name(prefix="mnq"),
                    protocol=protocol,
                    project_id=project_id,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Namespace(res.json())

    async def update_namespace(
        self,
        *,
        namespace_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
    ) -> Namespace:
        """
        Update the name of a namespace.
        Update the name of a Messaging and Queuing namespace, specified by its namespace ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: ID of the Namespace to update.
        :param name: Namespace name.
        :return: :class:`Namespace <Namespace>`

        Usage:
        ::

            result = await api.update_namespace(namespace_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "PATCH",
            f"/mnq/v1alpha1/regions/{param_region}/namespaces",
            body=marshal_UpdateNamespaceRequest(
                UpdateNamespaceRequest(
                    namespace_id=namespace_id,
                    region=region,
                    name=name,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Namespace(res.json())

    async def get_namespace(
        self,
        *,
        namespace_id: str,
        region: Optional[Region] = None,
    ) -> Namespace:
        """
        Get a namespace.
        Retrieve information about an existing Messaging and Queuing namespace, identified by its namespace ID. Its full details, including name, endpoint and protocol, are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: ID of the Namespace to get.
        :return: :class:`Namespace <Namespace>`

        Usage:
        ::

            result = await api.get_namespace(namespace_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_namespace_id = validate_path_param("namespace_id", namespace_id)

        res = self._request(
            "GET",
            f"/mnq/v1alpha1/regions/{param_region}/namespaces/{param_namespace_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Namespace(res.json())

    async def delete_namespace(
        self,
        *,
        namespace_id: str,
        region: Optional[Region] = None,
    ) -> Optional[None]:
        """
        Delete a namespace.
        Delete a Messaging and Queuing namespace, specified by its namespace ID. Note that deleting a namespace is irreversible, and any URLs, credentials and queued messages belonging to this namespace will also be deleted.
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: ID of the namespace to delete.

        Usage:
        ::

            result = await api.delete_namespace(namespace_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_namespace_id = validate_path_param("namespace_id", namespace_id)

        res = self._request(
            "DELETE",
            f"/mnq/v1alpha1/regions/{param_region}/namespaces/{param_namespace_id}",
        )

        self._throw_on_error(res)
        return None

    async def create_credential(
        self,
        *,
        namespace_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        permissions: Optional[Permissions] = None,
    ) -> Credential:
        """
        Create credentials.
        Create a set of credentials for a Messaging and Queuing namespace, specified by its namespace ID. If creating credentials for a NATS namespace, the `permissions` object must not be included in the request. If creating credentials for an SQS/SNS namespace, the `permissions` object is required, with all three of its child attributes.
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: Namespace containing the credentials.
        :param name: Name of the credentials.
        :param permissions: Permissions associated with these credentials.
        :return: :class:`Credential <Credential>`

        Usage:
        ::

            result = await api.create_credential(namespace_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/mnq/v1alpha1/regions/{param_region}/credentials",
            body=marshal_CreateCredentialRequest(
                CreateCredentialRequest(
                    namespace_id=namespace_id,
                    region=region,
                    name=name or random_name(prefix="mnq"),
                    permissions=permissions,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Credential(res.json())

    async def delete_credential(
        self,
        *,
        credential_id: str,
        region: Optional[Region] = None,
    ) -> Optional[None]:
        """
        Delete credentials.
        Delete a set of credentials, specified by their credential ID. Deleting credentials is irreversible and cannot be undone. The credentials can no longer be used to access the namespace.
        :param region: Region to target. If none is passed will use default region from the config.
        :param credential_id: ID of the credentials to delete.

        Usage:
        ::

            result = await api.delete_credential(credential_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_credential_id = validate_path_param("credential_id", credential_id)

        res = self._request(
            "DELETE",
            f"/mnq/v1alpha1/regions/{param_region}/credentials/{param_credential_id}",
        )

        self._throw_on_error(res)
        return None

    async def list_credentials(
        self,
        *,
        region: Optional[Region] = None,
        namespace_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: ListCredentialsRequestOrderBy = ListCredentialsRequestOrderBy.ID_ASC,
    ) -> ListCredentialsResponse:
        """
        List credentials.
        List existing credentials in the specified region. The response contains only the metadata for the credentials, not the credentials themselves (for this, use **Get Credentials**).
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: Namespace containing the credentials.
        :param page: Page number to return.
        :param page_size: Maximum number of credentials to return per page.
        :param order_by: Order in which to return results.
        :return: :class:`ListCredentialsResponse <ListCredentialsResponse>`

        Usage:
        ::

            result = await api.list_credentials()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/mnq/v1alpha1/regions/{param_region}/credentials",
            params={
                "namespace_id": namespace_id,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListCredentialsResponse(res.json())

    async def list_credentials_all(
        self,
        *,
        region: Optional[Region] = None,
        namespace_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListCredentialsRequestOrderBy] = None,
    ) -> List[CredentialSummary]:
        """
        List credentials.
        List existing credentials in the specified region. The response contains only the metadata for the credentials, not the credentials themselves (for this, use **Get Credentials**).
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: Namespace containing the credentials.
        :param page: Page number to return.
        :param page_size: Maximum number of credentials to return per page.
        :param order_by: Order in which to return results.
        :return: :class:`List[ListCredentialsResponse] <List[ListCredentialsResponse]>`

        Usage:
        ::

            result = await api.list_credentials_all()
        """

        return await fetch_all_pages_async(
            type=ListCredentialsResponse,
            key="credentials",
            fetcher=self.list_credentials,
            args={
                "region": region,
                "namespace_id": namespace_id,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
            },
        )

    async def update_credential(
        self,
        *,
        credential_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        permissions: Optional[Permissions] = None,
    ) -> Credential:
        """
        Update credentials.
        Update a set of credentials. You can update the credentials' name, or (in the case of SQS/SNS credentials only) their permissions. To update the name of NATS credentials, do not include the `permissions` object in your request.
        :param region: Region to target. If none is passed will use default region from the config.
        :param credential_id: ID of the credentials to update.
        :param name: Name of the credentials.
        :param permissions: Permissions associated with these credentials.
        :return: :class:`Credential <Credential>`

        Usage:
        ::

            result = await api.update_credential(credential_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_credential_id = validate_path_param("credential_id", credential_id)

        res = self._request(
            "PATCH",
            f"/mnq/v1alpha1/regions/{param_region}/credentials/{param_credential_id}",
            body=marshal_UpdateCredentialRequest(
                UpdateCredentialRequest(
                    credential_id=credential_id,
                    region=region,
                    name=name,
                    permissions=permissions,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Credential(res.json())

    async def get_credential(
        self,
        *,
        credential_id: str,
        region: Optional[Region] = None,
    ) -> Credential:
        """
        Get credentials.
        Retrieve an existing set of credentials, identified by the `credential_id`. The credentials themselves, as well as their metadata (protocol, namespace ID etc), are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param credential_id: ID of the credentials to get.
        :return: :class:`Credential <Credential>`

        Usage:
        ::

            result = await api.get_credential(credential_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_credential_id = validate_path_param("credential_id", credential_id)

        res = self._request(
            "GET",
            f"/mnq/v1alpha1/regions/{param_region}/credentials/{param_credential_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Credential(res.json())



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Region,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class ImageStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    READY = "ready"
    DELETING = "deleting"
    ERROR = "error"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class ImageVisibility(str, Enum, metaclass=StrEnumMeta):
    VISIBILITY_UNKNOWN = "visibility_unknown"
    INHERIT = "inherit"
    PUBLIC = "public"
    PRIVATE = "private"

    def __str__(self) -> str:
        return str(self.value)


class ListImagesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListNamespacesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    DESCRIPTION_ASC = "description_asc"
    DESCRIPTION_DESC = "description_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListTagsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class NamespaceStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    READY = "ready"
    DELETING = "deleting"
    ERROR = "error"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class TagStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    READY = "ready"
    DELETING = "deleting"
    ERROR = "error"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class Image:
    """
    Image.
    """

    id: str
    """
    UUID of the image.
    """

    name: str
    """
    Name of the image, it must be unique within the namespace.
    """

    namespace_id: str
    """
    UUID of the namespace the image belongs to.
    """

    status: ImageStatus
    """
    Status of the image.
    """

    status_message: Optional[str]
    """
    Details of the image status.
    """

    visibility: ImageVisibility
    """
    Set to `public` to allow the image to be pulled without authentication. Else, set to  `private`. Set to `inherit` to keep the same visibility configuration as the namespace.
    """

    size: int
    """
    Image size in bytes, calculated from the size of image layers.
    Image size in bytes, calculated from the size of image layers. One layer used in two tags of the same image is counted once but one layer used in two images is counted twice.
    """

    created_at: Optional[datetime]
    """
    Date and time of image creation.
    """

    updated_at: Optional[datetime]
    """
    Date and time of last update.
    """

    tags: List[str]
    """
    List of docker tags of the image.
    """


@dataclass
class ListImagesResponse:
    """
    List images response.
    """

    images: List[Image]
    """
    Paginated list of images that match the selected filters.
    """

    total_count: int
    """
    Total number of images that match the selected filters.
    """


@dataclass
class ListNamespacesResponse:
    """
    List namespaces response.
    """

    namespaces: List[Namespace]
    """
    Paginated list of namespaces that match the selected filters.
    """

    total_count: int
    """
    Total number of namespaces that match the selected filters.
    """


@dataclass
class ListTagsResponse:
    """
    List tags response.
    """

    tags: List[Tag]
    """
    Paginated list of tags that match the selected filters.
    """

    total_count: int
    """
    Total number of tags that match the selected filters.
    """


@dataclass
class Namespace:
    """
    Namespace.
    """

    id: str
    """
    UUID of the namespace.
    """

    name: str
    """
    Name of the namespace, unique in a region accross all organizations.
    """

    description: str
    """
    Description of the namespace.
    """

    organization_id: str
    """
    Owner of the namespace.
    """

    project_id: str
    """
    Project of the namespace.
    """

    status: NamespaceStatus
    """
    Namespace status.
    """

    status_message: str
    """
    Namespace status details.
    """

    endpoint: str
    """
    Endpoint reachable by docker.
    """

    is_public: bool
    """
    Defines whether or not namespace is public.
    """

    size: int
    """
    Total size of the namespace, calculated as the sum of the size of all images in the namespace.
    """

    created_at: Optional[datetime]
    """
    Date and time of creation.
    """

    updated_at: Optional[datetime]
    """
    Date and time of last update.
    """

    image_count: int
    """
    Number of images in the namespace.
    """

    region: Region
    """
    Region the namespace belongs to.
    """


@dataclass
class Tag:
    """
    Tag.
    """

    id: str
    """
    UUID of the tag.
    """

    name: str
    """
    Tag name, unique to an image.
    """

    image_id: str
    """
    Image ID the of the image the tag belongs to.
    """

    status: TagStatus
    """
    Tag status.
    """

    digest: str
    """
    Hash of the tag content. Several tags of a same image may have the same digest.
    """

    created_at: Optional[datetime]
    """
    Date and time of creation.
    """

    updated_at: Optional[datetime]
    """
    Date and time of last update.
    """


@dataclass
class ListNamespacesRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    page: Optional[int]
    """
    A positive integer to choose the page to display.
    """

    page_size: Optional[int]
    """
    A positive integer lower or equal to 100 to select the number of items to display.
    """

    order_by: Optional[ListNamespacesRequestOrderBy]
    """
    Criteria to use when ordering namespace listings. Possible values are `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`, `status_asc` and `status_desc`. The default value is `created_at_asc`.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID.
    """

    project_id: Optional[str]
    """
    Filter by Project ID.
    """

    name: Optional[str]
    """
    Filter by the namespace name (exact match).
    """


@dataclass
class GetNamespaceRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    namespace_id: str
    """
    UUID of the namespace.
    """


@dataclass
class CreateNamespaceRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name of the namespace.
    """

    description: str
    """
    Description of the namespace.
    """

    organization_id: Optional[str]
    """
    Namespace owner (deprecated).
    
    One-of ('project_identifier'): at most one of 'organization_id', 'project_id' could be set.
    :deprecated
    """

    project_id: Optional[str]
    """
    Project ID on which the namespace will be created.
    
    One-of ('project_identifier'): at most one of 'organization_id', 'project_id' could be set.
    """

    is_public: bool
    """
    Defines whether or not namespace is public.
    """


@dataclass
class UpdateNamespaceRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    namespace_id: str
    """
    ID of the namespace to update.
    """

    description: Optional[str]
    """
    Namespace description.
    """

    is_public: Optional[bool]
    """
    Defines whether or not the namespace is public.
    """


@dataclass
class DeleteNamespaceRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    namespace_id: str
    """
    UUID of the namespace.
    """


@dataclass
class ListImagesRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    page: Optional[int]
    """
    A positive integer to choose the page to display.
    """

    page_size: Optional[int]
    """
    A positive integer lower or equal to 100 to select the number of items to display.
    """

    order_by: Optional[ListImagesRequestOrderBy]
    """
    Criteria to use when ordering image listings. Possible values are `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`, `status_asc` and `status_desc`. The default value is `created_at_asc`.
    """

    namespace_id: Optional[str]
    """
    Filter by the namespace ID.
    """

    name: Optional[str]
    """
    Filter by the image name (exact match).
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID.
    """

    project_id: Optional[str]
    """
    Filter by Project ID.
    """


@dataclass
class GetImageRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    image_id: str
    """
    UUID of the image.
    """


@dataclass
class UpdateImageRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    image_id: str
    """
    ID of the image to update.
    """

    visibility: ImageVisibility
    """
    Set to `public` to allow the image to be pulled without authentication. Else, set to  `private`. Set to `inherit` to keep the same visibility configuration as the namespace.
    """


@dataclass
class DeleteImageRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    image_id: str
    """
    UUID of the image.
    """


@dataclass
class ListTagsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    image_id: str
    """
    UUID of the image.
    """

    page: Optional[int]
    """
    A positive integer to choose the page to display.
    """

    page_size: Optional[int]
    """
    A positive integer lower or equal to 100 to select the number of items to display.
    """

    order_by: Optional[ListTagsRequestOrderBy]
    """
    Criteria to use when ordering tag listings. Possible values are `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`, `status_asc` and `status_desc`. The default value is `created_at_asc`.
    """

    name: Optional[str]
    """
    Filter by the tag name (exact match).
    """


@dataclass
class GetTagRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    tag_id: str
    """
    UUID of the tag.
    """


@dataclass
class DeleteTagRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    tag_id: str
    """
    UUID of the tag.
    """

    force: Optional[bool]
    """
    If two tags share the same digest the deletion will fail unless this parameter is set to true (deprecated).
    :deprecated
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    ImageState,
    IpState,
    PrivateNICState,
    SecurityGroupState,
    ServerState,
    SnapshotState,
    TaskStatus,
    VolumeServerState,
    VolumeState,
)


IMAGE_TRANSIENT_STATUSES: List[ImageState] = [
    ImageState.CREATING,
]
"""
Lists transient statutes of the enum :class:`ImageState <ImageState>`.
"""

IP_TRANSIENT_STATUSES: List[IpState] = [
    IpState.PENDING,
]
"""
Lists transient statutes of the enum :class:`IpState <IpState>`.
"""

PRIVATE_NIC_TRANSIENT_STATUSES: List[PrivateNICState] = [
    PrivateNICState.SYNCING,
]
"""
Lists transient statutes of the enum :class:`PrivateNICState <PrivateNICState>`.
"""

SECURITY_GROUP_TRANSIENT_STATUSES: List[SecurityGroupState] = [
    SecurityGroupState.SYNCING,
]
"""
Lists transient statutes of the enum :class:`SecurityGroupState <SecurityGroupState>`.
"""

SERVER_TRANSIENT_STATUSES: List[ServerState] = [
    ServerState.STARTING,
    ServerState.STOPPING,
]
"""
Lists transient statutes of the enum :class:`ServerState <ServerState>`.
"""

SNAPSHOT_TRANSIENT_STATUSES: List[SnapshotState] = [
    SnapshotState.SNAPSHOTTING,
    SnapshotState.IMPORTING,
    SnapshotState.EXPORTING,
]
"""
Lists transient statutes of the enum :class:`SnapshotState <SnapshotState>`.
"""

TASK_TRANSIENT_STATUSES: List[TaskStatus] = [
    TaskStatus.PENDING,
    TaskStatus.STARTED,
    TaskStatus.RETRY,
]
"""
Lists transient statutes of the enum :class:`TaskStatus <TaskStatus>`.
"""

VOLUME_SERVER_TRANSIENT_STATUSES: List[VolumeServerState] = [
    VolumeServerState.SNAPSHOTTING,
    VolumeServerState.FETCHING,
    VolumeServerState.RESIZING,
    VolumeServerState.SAVING,
    VolumeServerState.HOTSYNCING,
]
"""
Lists transient statutes of the enum :class:`VolumeServerState <VolumeServerState>`.
"""

VOLUME_TRANSIENT_STATUSES: List[VolumeState] = [
    VolumeState.SNAPSHOTTING,
    VolumeState.FETCHING,
    VolumeState.RESIZING,
    VolumeState.SAVING,
    VolumeState.HOTSYNCING,
]
"""
Lists transient statutes of the enum :class:`VolumeState <VolumeState>`.
"""



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

from scaleway_core.bridge import (
    Zone,
)


@dataclass
class GetImageResponse:
    image: Optional[Image]


@dataclass
class GetVersionResponse:
    version: Optional[Version]


@dataclass
class Image:
    """
    Image.
    """

    id: str
    """
    UUID of this image.
    """

    name: str
    """
    Name of the image.
    """

    description: str
    """
    Text description of this image.
    """

    logo: str
    """
    URL of this image's logo.
    """

    categories: List[str]
    """
    List of categories this image belongs to.
    """

    creation_date: Optional[datetime]
    """
    Creation date of this image.
    """

    modification_date: Optional[datetime]
    """
    Date of the last modification of this image.
    """

    valid_until: Optional[datetime]
    """
    Expiration date of this image.
    """

    label: str
    """
    Label of this image.
    Typically an identifier for a distribution (ex. "ubuntu_focal").
    """

    versions: List[Version]
    """
    List of versions of this image.
    """

    organization: Optional[Organization]
    """
    Organization this image belongs to.
    """

    current_public_version: str


@dataclass
class ListImagesResponse:
    images: List[Image]

    total_count: int


@dataclass
class ListVersionsResponse:
    versions: List[Version]

    total_count: int


@dataclass
class LocalImage:
    """
    Local image.
    """

    id: str
    """
    UUID of this local image.
    Version you will typically use to define an image in an API call.
    """

    compatible_commercial_types: List[str]
    """
    List of all commercial types that are compatible with this local image.
    """

    arch: str
    """
    Supported architecture for this local image.
    """

    zone: Zone
    """
    Availability Zone where this local image is available.
    """


@dataclass
class Organization:
    id: str

    name: str


@dataclass
class Version:
    """
    Version.
    """

    id: str
    """
    UUID of this version.
    """

    name: str
    """
    Name of this version.
    """

    creation_date: Optional[datetime]
    """
    Creation date of this image version.
    """

    modification_date: Optional[datetime]
    """
    Date of the last modification of this version.
    """

    local_images: List[LocalImage]
    """
    List of local images available in this version.
    """


@dataclass
class ListImagesRequest:
    per_page: Optional[int]
    """
    A positive integer lower or equal to 100 to select the number of items to display.
    """

    page: Optional[int]
    """
    A positive integer to choose the page to display.
    """


@dataclass
class GetImageRequest:
    image_id: str
    """
    Display the image name.
    """


@dataclass
class ListVersionsRequest:
    image_id: str


@dataclass
class GetVersionRequest:
    image_id: str

    version_id: str



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Zone,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class DHCPEntryType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    RESERVATION = "reservation"
    LEASE = "lease"

    def __str__(self) -> str:
        return str(self.value)


class GatewayNetworkStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    CREATED = "created"
    ATTACHING = "attaching"
    CONFIGURING = "configuring"
    READY = "ready"
    DETACHING = "detaching"
    DELETED = "deleted"

    def __str__(self) -> str:
        return str(self.value)


class GatewayStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    STOPPED = "stopped"
    ALLOCATING = "allocating"
    CONFIGURING = "configuring"
    RUNNING = "running"
    STOPPING = "stopping"
    FAILED = "failed"
    DELETING = "deleting"
    DELETED = "deleted"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class ListDHCPEntriesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    IP_ADDRESS_ASC = "ip_address_asc"
    IP_ADDRESS_DESC = "ip_address_desc"
    HOSTNAME_ASC = "hostname_asc"
    HOSTNAME_DESC = "hostname_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListDHCPsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    SUBNET_ASC = "subnet_asc"
    SUBNET_DESC = "subnet_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListGatewayNetworksRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    STATUS_ASC = "status_asc"
    STATUS_DESC = "status_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListGatewaysRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    TYPE_ASC = "type_asc"
    TYPE_DESC = "type_desc"
    STATUS_ASC = "status_asc"
    STATUS_DESC = "status_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListIPsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    IP_ASC = "ip_asc"
    IP_DESC = "ip_desc"
    REVERSE_ASC = "reverse_asc"
    REVERSE_DESC = "reverse_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListPATRulesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    PUBLIC_PORT_ASC = "public_port_asc"
    PUBLIC_PORT_DESC = "public_port_desc"

    def __str__(self) -> str:
        return str(self.value)


class PATRuleProtocol(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    BOTH = "both"
    TCP = "tcp"
    UDP = "udp"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class DHCP:
    """
    Dhcp.
    """

    id: str
    """
    ID of the DHCP config.
    """

    organization_id: str
    """
    Owning Organization.
    """

    project_id: str
    """
    Owning Project.
    """

    created_at: Optional[datetime]
    """
    Date the DHCP configuration was created.
    """

    updated_at: Optional[datetime]
    """
    Configuration last modification date.
    """

    subnet: str
    """
    Subnet for the DHCP server.
    """

    address: str
    """
    IP address of the DHCP server. This will be the Public Gateway's address in the Private Network. It must be part of config's subnet.
    """

    pool_low: str
    """
    Low IP (inclusive) of the dynamic address pool. Must be in the config's subnet.
    """

    pool_high: str
    """
    High IP (inclusive) of the dynamic address pool. Must be in the config's subnet.
    """

    enable_dynamic: bool
    """
    Defines whether to enable dynamic pooling of IPs. When false, only pre-existing DHCP reservations will be handed out.
    """

    valid_lifetime: Optional[str]
    """
    How long DHCP entries will be valid for.
    """

    renew_timer: Optional[str]
    """
    After how long a renew will be attempted. Must be 30s lower than `rebind_timer`.
    """

    rebind_timer: Optional[str]
    """
    After how long a DHCP client will query for a new lease if previous renews fail. Must be 30s lower than `valid_lifetime`.
    """

    push_default_route: bool
    """
    Defines whether the gateway should push a default route to DHCP clients, or only hand out IPs.
    """

    push_dns_server: bool
    """
    Defines whether the gateway should push custom DNS servers to clients. This allows for instance hostname -> IP resolution.
    """

    dns_servers_override: List[str]
    """
    Array of DNS server IP addresses used to override the DNS server list pushed to DHCP clients, instead of the gateway itself.
    """

    dns_search: List[str]
    """
    Array of search paths in addition to the pushed DNS configuration.
    """

    dns_local_name: str
    """
    TLD given to hostnames in the Private Networks. If an Instance with hostname `foo` gets a lease, and this is set to `bar`, `foo.bar` will resolve.
    """

    zone: Zone
    """
    Zone of this DHCP configuration.
    """


@dataclass
class DHCPEntry:
    """
    Dhcp entry.
    """

    id: str
    """
    DHCP entry ID.
    """

    created_at: Optional[datetime]
    """
    DHCP entry creation date.
    """

    updated_at: Optional[datetime]
    """
    DHCP entry last modification date.
    """

    gateway_network_id: str
    """
    Owning GatewayNetwork.
    """

    mac_address: str
    """
    MAC address of the client device.
    """

    ip_address: str
    """
    Assigned IP address.
    """

    hostname: str
    """
    Hostname of the client device.
    """

    type_: DHCPEntryType
    """
    Entry type, either static (DHCP reservation) or dynamic (DHCP lease).
    """

    zone: Zone
    """
    Zone of this DHCP entry.
    """


@dataclass
class Gateway:
    """
    Gateway.
    """

    id: str
    """
    ID of the gateway.
    """

    organization_id: str
    """
    Owning Organization.
    """

    project_id: str
    """
    Owning Project.
    """

    created_at: Optional[datetime]
    """
    Gateway creation date.
    """

    updated_at: Optional[datetime]
    """
    Gateway last modification date.
    """

    type_: Optional[GatewayType]
    """
    Gateway type (commercial offer).
    """

    status: GatewayStatus
    """
    Current status of the gateway.
    """

    name: str
    """
    Name of the gateway.
    """

    tags: List[str]
    """
    Tags associated with the gateway.
    """

    ip: Optional[IP]
    """
    Public IP address of the gateway.
    """

    gateway_networks: List[GatewayNetwork]
    """
    GatewayNetwork objects attached to the gateway (each one represents a connection to a Private Network).
    """

    upstream_dns_servers: List[str]
    """
    Array of DNS server IP addresses to override the gateway's default recursive DNS servers.
    """

    version: Optional[str]
    """
    Version of the running gateway software.
    """

    can_upgrade_to: Optional[str]
    """
    Newly available gateway software version that can be updated to.
    """

    bastion_enabled: bool
    """
    Defines whether SSH bastion is enabled on the gateway.
    """

    bastion_port: int
    """
    Port of the SSH bastion.
    """

    smtp_enabled: bool
    """
    Defines whether SMTP traffic is allowed to pass through the gateway.
    """

    zone: Zone
    """
    Zone of the gateway.
    """


@dataclass
class GatewayNetwork:
    """
    Gateway network.
    """

    id: str
    """
    ID of the Public Gateway-Private Network connection.
    """

    created_at: Optional[datetime]
    """
    Connection creation date.
    """

    updated_at: Optional[datetime]
    """
    Connection last modification date.
    """

    gateway_id: str
    """
    ID of the connected Public Gateway.
    """

    private_network_id: str
    """
    ID of the connected Private Network.
    """

    mac_address: Optional[str]
    """
    MAC address of the gateway in the Private Network (if the gateway is up and running).
    """

    enable_masquerade: bool
    """
    Defines whether the gateway masquerades traffic for this Private Network (Dynamic NAT).
    """

    status: GatewayNetworkStatus
    """
    Current status of the Public Gateway's connection to the Private Network.
    """

    dhcp: Optional[DHCP]
    """
    DHCP configuration for the connected Private Network.
    """

    enable_dhcp: bool
    """
    Defines whether DHCP is enabled on the connected Private Network.
    """

    address: Optional[str]
    """
    Address of the Gateway (in CIDR form) to use when DHCP is not used.
    """

    zone: Zone
    """
    Zone of the GatewayNetwork connection.
    """


@dataclass
class GatewayType:
    """
    Gateway type.
    """

    name: str
    """
    Public Gateway type name.
    """

    bandwidth: int
    """
    Bandwidth, in bps, of the Public Gateway. This is the public bandwidth to the outer Internet, and the internal bandwidth to each connected Private Networks.
    """

    zone: Zone
    """
    Zone the Public Gateway type is available in.
    """


@dataclass
class IP:
    """
    Ip.
    """

    id: str
    """
    IP address ID.
    """

    organization_id: str
    """
    Owning Organization.
    """

    project_id: str
    """
    Owning Project.
    """

    created_at: Optional[datetime]
    """
    IP address creation date.
    """

    updated_at: Optional[datetime]
    """
    IP address last modification date.
    """

    tags: List[str]
    """
    Tags associated with the IP address.
    """

    address: str
    """
    The IP address itself.
    """

    reverse: Optional[str]
    """
    Reverse domain name for the IP address.
    """

    gateway_id: Optional[str]
    """
    Public Gateway associated with the IP address.
    """

    zone: Zone
    """
    Zone of the IP address.
    """


@dataclass
class ListDHCPEntriesResponse:
    """
    List dhcp entries response.
    """

    dhcp_entries: List[DHCPEntry]
    """
    DHCP entries in this page.
    """

    total_count: int
    """
    Total count of DHCP entries matching the filter.
    """


@dataclass
class ListDHCPsResponse:
    """
    List dhc ps response.
    """

    dhcps: List[DHCP]
    """
    First page of DHCP configuration objects.
    """

    total_count: int
    """
    Total count of DHCP configuration objects matching the filter.
    """


@dataclass
class ListGatewayNetworksResponse:
    """
    List gateway networks response.
    """

    gateway_networks: List[GatewayNetwork]
    """
    GatewayNetworks on this page.
    """

    total_count: int
    """
    Total GatewayNetworks count matching the filter.
    """


@dataclass
class ListGatewayTypesResponse:
    """
    List gateway types response.
    """

    types: List[GatewayType]
    """
    Available types of Public Gateway.
    """


@dataclass
class ListGatewaysResponse:
    """
    List gateways response.
    """

    gateways: List[Gateway]
    """
    Gateways on this page.
    """

    total_count: int
    """
    Total count of gateways matching the filter.
    """


@dataclass
class ListIPsResponse:
    """
    List i ps response.
    """

    ips: List[IP]
    """
    IP addresses on this page.
    """

    total_count: int
    """
    Total count of IP addresses matching the filter.
    """


@dataclass
class ListPATRulesResponse:
    """
    List pat rules response.
    """

    pat_rules: List[PATRule]
    """
    Array of PAT rules matching the filter.
    """

    total_count: int
    """
    Total count of PAT rules matching the filter.
    """


@dataclass
class PATRule:
    """
    Pat rule.
    """

    id: str
    """
    PAT rule ID.
    """

    gateway_id: str
    """
    Gateway the PAT rule applies to.
    """

    created_at: Optional[datetime]
    """
    PAT rule creation date.
    """

    updated_at: Optional[datetime]
    """
    PAT rule last modification date.
    """

    public_port: int
    """
    Public port to listen on.
    """

    private_ip: str
    """
    Private IP address to forward data to.
    """

    private_port: int
    """
    Private port to translate to.
    """

    protocol: PATRuleProtocol
    """
    Protocol the rule applies to.
    """

    zone: Zone
    """
    Zone of the PAT rule.
    """


@dataclass
class SetDHCPEntriesRequestEntry:
    """
    Set dhcp entries request. entry.
    """

    mac_address: str
    """
    MAC address to give a static entry to.
    MAC address to give a static entry to. A matching entry will be upgraded to a reservation, and a matching reservation will be updated.
    """

    ip_address: str
    """
    IP address to give to the device.
    """


@dataclass
class SetDHCPEntriesResponse:
    """
    Set dhcp entries response.
    """

    dhcp_entries: List[DHCPEntry]
    """
    List of DHCP entries.
    """


@dataclass
class SetPATRulesRequestRule:
    """
    Set pat rules request. rule.
    """

    public_port: int
    """
    Public port to listen on.
    Public port to listen on. Uniquely identifies the rule, and a matching rule will be updated with the new parameters.
    """

    private_ip: str
    """
    Private IP to forward data to.
    """

    private_port: int
    """
    Private port to translate to.
    """

    protocol: PATRuleProtocol
    """
    Protocol the rule should apply to.
    """


@dataclass
class SetPATRulesResponse:
    """
    Set pat rules response.
    """

    pat_rules: List[PATRule]
    """
    List of PAT rules.
    """


@dataclass
class ListGatewaysRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListGatewaysRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number to return.
    """

    page_size: Optional[int]
    """
    Gateways per page.
    """

    organization_id: Optional[str]
    """
    Include only gateways in this Organization.
    """

    project_id: Optional[str]
    """
    Include only gateways in this Project.
    """

    name: Optional[str]
    """
    Filter for gateways which have this search term in their name.
    """

    tags: Optional[List[str]]
    """
    Filter for gateways with these tags.
    """

    type_: Optional[str]
    """
    Filter for gateways of this type.
    """

    status: Optional[GatewayStatus]
    """
    Filter for gateways with this current status. Use `unknown` to include all statuses.
    """

    private_network_id: Optional[str]
    """
    Filter for gateways attached to this Private nNetwork.
    """


@dataclass
class GetGatewayRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_id: str
    """
    ID of the gateway to fetch.
    """


@dataclass
class CreateGatewayRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Scaleway Project to create the gateway in.
    """

    name: Optional[str]
    """
    Name for the gateway.
    """

    tags: Optional[List[str]]
    """
    Tags for the gateway.
    """

    type_: str
    """
    Gateway type (commercial offer type).
    """

    upstream_dns_servers: Optional[List[str]]
    """
    Array of DNS server IP addresses to override the gateway's default recursive DNS servers.
    """

    ip_id: Optional[str]
    """
    Existing IP address to attach to the gateway.
    """

    enable_smtp: bool
    """
    Defines whether SMTP traffic should be allowed pass through the gateway.
    """

    enable_bastion: bool
    """
    Defines whether SSH bastion should be enabled the gateway.
    """

    bastion_port: Optional[int]
    """
    Port of the SSH bastion.
    """


@dataclass
class UpdateGatewayRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_id: str
    """
    ID of the gateway to update.
    """

    name: Optional[str]
    """
    Name for the gateway.
    """

    tags: Optional[List[str]]
    """
    Tags for the gateway.
    """

    upstream_dns_servers: Optional[List[str]]
    """
    Array of DNS server IP addresses to override the gateway's default recursive DNS servers.
    """

    enable_bastion: Optional[bool]
    """
    Defines whether SSH bastion should be enabled the gateway.
    """

    bastion_port: Optional[int]
    """
    Port of the SSH bastion.
    """

    enable_smtp: Optional[bool]
    """
    Defines whether SMTP traffic should be allowed to pass through the gateway.
    """


@dataclass
class DeleteGatewayRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_id: str
    """
    ID of the gateway to delete.
    """

    cleanup_dhcp: bool
    """
    Defines whether to clean up attached DHCP configurations (if any, and if not attached to another Gateway Network).
    """


@dataclass
class UpgradeGatewayRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_id: str
    """
    ID of the gateway to upgrade.
    """


@dataclass
class ListGatewayNetworksRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListGatewayNetworksRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    GatewayNetworks per page.
    """

    gateway_id: Optional[str]
    """
    Filter for GatewayNetworks connected to this gateway.
    """

    private_network_id: Optional[str]
    """
    Filter for GatewayNetworks connected to this Private Network.
    """

    enable_masquerade: Optional[bool]
    """
    Filter for GatewayNetworks with this `enable_masquerade` setting.
    """

    dhcp_id: Optional[str]
    """
    Filter for GatewayNetworks using this DHCP configuration.
    """

    status: Optional[GatewayNetworkStatus]
    """
    Filter for GatewayNetworks with this current status this status. Use `unknown` to include all statuses.
    """


@dataclass
class GetGatewayNetworkRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_network_id: str
    """
    ID of the GatewayNetwork to fetch.
    """


@dataclass
class CreateGatewayNetworkRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_id: str
    """
    Public Gateway to connect.
    """

    private_network_id: str
    """
    Private Network to connect.
    """

    enable_masquerade: bool
    """
    Defines whether to enable masquerade (dynamic NAT) on this network.
    """

    dhcp_id: Optional[str]
    """
    ID of an existing DHCP configuration object to use for this GatewayNetwork.
    
    One-of ('ip_config'): at most one of 'dhcp_id', 'dhcp', 'address' could be set.
    """

    dhcp: Optional[CreateDHCPRequest]
    """
    New DHCP configuration object to use for this GatewayNetwork.
    
    One-of ('ip_config'): at most one of 'dhcp_id', 'dhcp', 'address' could be set.
    """

    address: Optional[str]
    """
    Static IP address in CIDR format to to use without DHCP.
    
    One-of ('ip_config'): at most one of 'dhcp_id', 'dhcp', 'address' could be set.
    """

    enable_dhcp: Optional[bool]
    """
    Defines whether to enable DHCP on this Private Network. Defaults to `true` if either `dhcp_id` or `dhcp` are present. If set to `true`, either `dhcp_id` or `dhcp` must be present.
    """


@dataclass
class UpdateGatewayNetworkRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_network_id: str
    """
    ID of the GatewayNetwork to update.
    """

    enable_masquerade: Optional[bool]
    """
    Defines whether to enable masquerade (dynamic NAT) on the GatewayNetwork.
    """

    dhcp_id: Optional[str]
    """
    ID of the new DHCP configuration object to use with this GatewayNetwork.
    
    One-of ('ip_config'): at most one of 'dhcp_id', 'address' could be set.
    """

    enable_dhcp: Optional[bool]
    """
    Defines whether to enable DHCP on the connected Private Network.
    """

    address: Optional[str]
    """
    New static IP address.
    
    One-of ('ip_config'): at most one of 'dhcp_id', 'address' could be set.
    """


@dataclass
class DeleteGatewayNetworkRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_network_id: str
    """
    ID of the GatewayNetwork to delete.
    """

    cleanup_dhcp: bool
    """
    Defines whether to clean up attached DHCP configurations (if any, and if not attached to another Gateway Network).
    """


@dataclass
class ListDHCPsRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListDHCPsRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    DHCP configurations per page.
    """

    organization_id: Optional[str]
    """
    Include only DHCP configuration objects in this Organization.
    """

    project_id: Optional[str]
    """
    Include only DHCP configuration objects in this Project.
    """

    address: Optional[str]
    """
    Filter for DHCP configuration objects with this DHCP server IP address (the gateway's address in the Private Network).
    """

    has_address: Optional[str]
    """
    Filter for DHCP configuration objects with subnets containing this IP address.
    """


@dataclass
class GetDHCPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    dhcp_id: str
    """
    ID of the DHCP configuration to fetch.
    """


@dataclass
class CreateDHCPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project to create the DHCP configuration in.
    """

    subnet: str
    """
    Subnet for the DHCP server.
    """

    address: Optional[str]
    """
    IP address of the DHCP server. This will be the gateway's address in the Private Network. Defaults to the first address of the subnet.
    """

    pool_low: Optional[str]
    """
    Low IP (inclusive) of the dynamic address pool. Must be in the config's subnet. Defaults to the second address of the subnet.
    """

    pool_high: Optional[str]
    """
    High IP (inclusive) of the dynamic address pool. Must be in the config's subnet. Defaults to the last address of the subnet.
    """

    enable_dynamic: Optional[bool]
    """
    Defines whether to enable dynamic pooling of IPs. When false, only pre-existing DHCP reservations will be handed out. Defaults to true.
    """

    valid_lifetime: Optional[str]
    """
    How long DHCP entries will be valid for. Defaults to 1h (3600s).
    """

    renew_timer: Optional[str]
    """
    After how long a renew will be attempted. Must be 30s lower than `rebind_timer`. Defaults to 50m (3000s).
    """

    rebind_timer: Optional[str]
    """
    After how long a DHCP client will query for a new lease if previous renews fail. Must be 30s lower than `valid_lifetime`. Defaults to 51m (3060s).
    """

    push_default_route: Optional[bool]
    """
    Defines whether the gateway should push a default route to DHCP clients or only hand out IPs. Defaults to true.
    """

    push_dns_server: Optional[bool]
    """
    Defines whether the gateway should push custom DNS servers to clients. This allows for Instance hostname -> IP resolution. Defaults to true.
    """

    dns_servers_override: Optional[List[str]]
    """
    Array of DNS server IP addresses used to override the DNS server list pushed to DHCP clients, instead of the gateway itself.
    """

    dns_search: Optional[List[str]]
    """
    Array of search paths in addition to the pushed DNS configuration.
    """

    dns_local_name: Optional[str]
    """
    TLD given to hostnames in the Private Network. Allowed characters are `a-z0-9-.`. Defaults to the slugified Private Network name if created along a GatewayNetwork, or else to `priv`.
    """


@dataclass
class UpdateDHCPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    dhcp_id: str
    """
    DHCP configuration to update.
    """

    subnet: Optional[str]
    """
    Subnet for the DHCP server.
    """

    address: Optional[str]
    """
    IP address of the DHCP server. This will be the Public Gateway's address in the Private Network. It must be part of config's subnet.
    """

    pool_low: Optional[str]
    """
    Low IP (inclusive) of the dynamic address pool. Must be in the config's subnet.
    """

    pool_high: Optional[str]
    """
    High IP (inclusive) of the dynamic address pool. Must be in the config's subnet.
    """

    enable_dynamic: Optional[bool]
    """
    Defines whether to enable dynamic pooling of IPs. When false, only pre-existing DHCP reservations will be handed out. Defaults to true.
    """

    valid_lifetime: Optional[str]
    """
    How long DHCP entries will be valid for.
    """

    renew_timer: Optional[str]
    """
    After how long a renew will be attempted. Must be 30s lower than `rebind_timer`.
    """

    rebind_timer: Optional[str]
    """
    After how long a DHCP client will query for a new lease if previous renews fail. Must be 30s lower than `valid_lifetime`.
    """

    push_default_route: Optional[bool]
    """
    Defines whether the gateway should push a default route to DHCP clients, or only hand out IPs.
    """

    push_dns_server: Optional[bool]
    """
    Defines whether the gateway should push custom DNS servers to clients. This allows for instance hostname -> IP resolution.
    """

    dns_servers_override: Optional[List[str]]
    """
    Array of DNS server IP addresses used to override the DNS server list pushed to DHCP clients, instead of the gateway itself.
    """

    dns_search: Optional[List[str]]
    """
    Array of search paths in addition to the pushed DNS configuration.
    """

    dns_local_name: Optional[str]
    """
    TLD given to hostnames in the Private Networks. If an instance with hostname `foo` gets a lease, and this is set to `bar`, `foo.bar` will resolve. Allowed characters are `a-z0-9-.`.
    """


@dataclass
class DeleteDHCPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    dhcp_id: str
    """
    DHCP configuration ID to delete.
    """


@dataclass
class ListDHCPEntriesRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListDHCPEntriesRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    DHCP entries per page.
    """

    gateway_network_id: Optional[str]
    """
    Filter for entries on this GatewayNetwork.
    """

    mac_address: Optional[str]
    """
    Filter for entries with this MAC address.
    """

    ip_address: Optional[str]
    """
    Filter for entries with this IP address.
    """

    hostname: Optional[str]
    """
    Filter for entries with this hostname substring.
    """

    type_: Optional[DHCPEntryType]
    """
    Filter for entries of this type.
    """


@dataclass
class GetDHCPEntryRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    dhcp_entry_id: str
    """
    ID of the DHCP entry to fetch.
    """


@dataclass
class CreateDHCPEntryRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_network_id: str
    """
    GatewayNetwork on which to create a DHCP reservation.
    """

    mac_address: str
    """
    MAC address to give a static entry to.
    """

    ip_address: str
    """
    IP address to give to the device.
    """


@dataclass
class UpdateDHCPEntryRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    dhcp_entry_id: str
    """
    ID of the DHCP entry to update.
    """

    ip_address: Optional[str]
    """
    New IP address to give to the device.
    """


@dataclass
class SetDHCPEntriesRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_network_id: str
    """
    ID of the Gateway Network on which to set DHCP reservation list.
    """

    dhcp_entries: Optional[List[SetDHCPEntriesRequestEntry]]
    """
    New list of DHCP reservations.
    """


@dataclass
class DeleteDHCPEntryRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    dhcp_entry_id: str
    """
    ID of the DHCP entry to delete.
    """


@dataclass
class ListPATRulesRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListPATRulesRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    PAT rules per page.
    """

    gateway_id: Optional[str]
    """
    Filter for PAT rules on this Gateway.
    """

    private_ip: Optional[str]
    """
    Filter for PAT rules targeting this private ip.
    """

    protocol: Optional[PATRuleProtocol]
    """
    Filter for PAT rules with this protocol.
    """


@dataclass
class GetPATRuleRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    pat_rule_id: str
    """
    ID of the PAT rule to get.
    """


@dataclass
class CreatePATRuleRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_id: str
    """
    ID of the Gateway on which to create the rule.
    """

    public_port: int
    """
    Public port to listen on.
    """

    private_ip: str
    """
    Private IP to forward data to.
    """

    private_port: int
    """
    Private port to translate to.
    """

    protocol: PATRuleProtocol
    """
    Protocol the rule should apply to.
    """


@dataclass
class UpdatePATRuleRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    pat_rule_id: str
    """
    ID of the PAT rule to update.
    """

    public_port: Optional[int]
    """
    Public port to listen on.
    """

    private_ip: Optional[str]
    """
    Private IP to forward data to.
    """

    private_port: Optional[int]
    """
    Private port to translate to.
    """

    protocol: PATRuleProtocol
    """
    Protocol the rule should apply to.
    """


@dataclass
class SetPATRulesRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_id: str
    """
    ID of the gateway on which to set the PAT rules.
    """

    pat_rules: List[SetPATRulesRequestRule]
    """
    New list of PAT rules.
    """


@dataclass
class DeletePATRuleRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    pat_rule_id: str
    """
    ID of the PAT rule to delete.
    """


@dataclass
class ListGatewayTypesRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ListIPsRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListIPsRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    IP addresses per page.
    """

    organization_id: Optional[str]
    """
    Filter for IP addresses in this Organization.
    """

    project_id: Optional[str]
    """
    Filter for IP addresses in this Project.
    """

    tags: Optional[List[str]]
    """
    Filter for IP addresses with these tags.
    """

    reverse: Optional[str]
    """
    Filter for IP addresses that have a reverse containing this string.
    """

    is_free: Optional[bool]
    """
    Filter based on whether the IP is attached to a gateway or not.
    """


@dataclass
class GetIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    ip_id: str
    """
    ID of the IP address to get.
    """


@dataclass
class CreateIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project to create the IP address in.
    """

    tags: Optional[List[str]]
    """
    Tags to give to the IP address.
    """


@dataclass
class UpdateIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    ip_id: str
    """
    ID of the IP address to update.
    """

    tags: Optional[List[str]]
    """
    Tags to give to the IP address.
    """

    reverse: Optional[str]
    """
    Reverse to set on the address. Empty string to unset.
    """

    gateway_id: Optional[str]
    """
    Gateway to attach the IP address to. Empty string to detach.
    """


@dataclass
class DeleteIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    ip_id: str
    """
    ID of the IP address to delete.
    """


@dataclass
class RefreshSSHKeysRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    gateway_id: str
    """
    ID of the gateway to refresh SSH keys on.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Zone,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class ListImagesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListLocalImagesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListVersionsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class LocalImageType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_TYPE = "unknown_type"
    INSTANCE_LOCAL = "instance_local"
    INSTANCE_SBS = "instance_sbs"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class Category:
    id: str

    name: str

    description: str


@dataclass
class Image:
    """
    Image.
    """

    id: str
    """
    UUID of this image.
    """

    name: str
    """
    Name of the image.
    """

    description: str
    """
    Text description of this image.
    """

    logo: str
    """
    URL of this image's logo.
    """

    categories: List[str]
    """
    List of categories this image belongs to.
    """

    created_at: Optional[datetime]
    """
    Creation date of this image.
    """

    updated_at: Optional[datetime]
    """
    Date of the last modification of this image.
    """

    valid_until: Optional[datetime]
    """
    Expiration date of this image.
    """

    label: str
    """
    Label of this image.
    Typically an identifier for a distribution (ex. "ubuntu_focal").
    """


@dataclass
class ListCategoriesResponse:
    categories: List[Category]

    total_count: int


@dataclass
class ListImagesResponse:
    images: List[Image]

    total_count: int


@dataclass
class ListLocalImagesResponse:
    local_images: List[LocalImage]

    total_count: int


@dataclass
class ListVersionsResponse:
    versions: List[Version]

    total_count: int


@dataclass
class LocalImage:
    """
    Local image.
    """

    id: str
    """
    UUID of this local image.
    Version you will typically use to define an image in an API call.
    """

    compatible_commercial_types: List[str]
    """
    List of all commercial types that are compatible with this local image.
    """

    arch: str
    """
    Supported architecture for this local image.
    """

    zone: Zone
    """
    Availability Zone where this local image is available.
    """

    label: str
    """
    Image label this image belongs to.
    """

    type_: LocalImageType
    """
    Type of this local image.
    """


@dataclass
class Version:
    """
    Version.
    """

    id: str
    """
    UUID of this version.
    """

    name: str
    """
    Name of this version.
    """

    created_at: Optional[datetime]
    """
    Creation date of this image version.
    """

    updated_at: Optional[datetime]
    """
    Date of the last modification of this version.
    """

    published_at: Optional[datetime]
    """
    Date this version was officially published.
    """


@dataclass
class ListImagesRequest:
    page_size: Optional[int]
    """
    A positive integer lower or equal to 100 to select the number of items to display.
    """

    page: Optional[int]
    """
    A positive integer to choose the page to display.
    """

    order_by: Optional[ListImagesRequestOrderBy]
    """
    Ordering to use.
    """

    arch: Optional[str]
    """
    Choose for which machine architecture to return images.
    """

    category: Optional[str]
    """
    Choose the category of images to get.
    """

    include_eol: bool
    """
    Choose to include end-of-life images.
    """


@dataclass
class GetImageRequest:
    image_id: str
    """
    Display the image name.
    """


@dataclass
class ListVersionsRequest:
    image_id: str

    page_size: Optional[int]

    page: Optional[int]

    order_by: Optional[ListVersionsRequestOrderBy]


@dataclass
class GetVersionRequest:
    version_id: str


@dataclass
class ListLocalImagesRequest:
    image_id: Optional[str]
    """
    One-of ('scope'): at most one of 'image_id', 'version_id', 'image_label' could be set.
    """

    version_id: Optional[str]
    """
    One-of ('scope'): at most one of 'image_id', 'version_id', 'image_label' could be set.
    """

    page_size: Optional[int]

    page: Optional[int]

    order_by: Optional[ListLocalImagesRequestOrderBy]

    image_label: Optional[str]
    """
    One-of ('scope'): at most one of 'image_id', 'version_id', 'image_label' could be set.
    """

    zone: Optional[Zone]

    type_: Optional[LocalImageType]


@dataclass
class GetLocalImageRequest:
    local_image_id: str


@dataclass
class ListCategoriesRequest:
    page_size: Optional[int]

    page: Optional[int]


@dataclass
class GetCategoryRequest:
    category_id: str
